# Copyright (c) Alibaba, Inc. and its affiliates.

from copy import deepcopy
from typing import Any, Dict, List, Literal, Optional, Tuple, Union

from torch import Tensor
from transformers import PreTrainedTokenizerBase, StoppingCriteria

DEFAULT_SYSTEM = 'You are a helpful assistant.'  # qwen system
History = List[Union[Tuple[str, str], List[str]]]


class TemplateType:
    # text-generation
    default_generation = 'default-generation'
    default_generation_bos = 'default-generation-bos'
    chatglm_generation = 'chatglm-generation'
    # chat
    default = 'default'
    chatml = 'chatml'
    qwen = chatml
    baichuan = 'baichuan'
    chatglm2 = 'chatglm2'
    chatglm3 = 'chatglm3'
    llama = 'llama'
    openbuddy = 'openbuddy'
    internlm = 'internlm'
    yi = 'yi'
    xverse = 'xverse'
    ziya = 'ziya'
    skywork = 'skywork'
    bluelm = 'bluelm'
    zephyr = 'zephyr'


class ExtraProcessTokenizer:

    @staticmethod
    def qwen_base(tokenizer: PreTrainedTokenizerBase) -> PreTrainedTokenizerBase:
        tokenizer.eos_token_id = tokenizer.eod_id
        return tokenizer

    @staticmethod
    def qwen_chat(tokenizer: PreTrainedTokenizerBase) -> PreTrainedTokenizerBase:
        tokenizer.eos_token_id = tokenizer.im_end_id
        return tokenizer

    @staticmethod
    def internlm_chat(tokenizer: PreTrainedTokenizerBase) -> PreTrainedTokenizerBase:
        del tokenizer.__class__.eos_token_id
        tokenizer.eos_token = '<eoa>'
        return tokenizer

    @staticmethod
    def yi_chat(tokenizer: PreTrainedTokenizerBase) -> PreTrainedTokenizerBase:
        tokenizer.eos_token = '<|im_end|>'
        return tokenizer

    @staticmethod
    def skywork_chat(tokenizer: PreTrainedTokenizerBase) -> PreTrainedTokenizerBase:
        tokenizer.add_tokens('[USER]')
        tokenizer.add_tokens('[BOT]')
        tokenizer.add_tokens('[SEP]')
        return tokenizer


MODEL_TEMPLATE_MAP = {'baichuan-7B': ['default-generation', None],
                      'Baichuan-13B-Chat': ['baichuan', None],
                      'XVERSE-7B': ['default-generation', None],
                      'XVERSE-7B-Chat': ['xverse', None],
                      'XVERSE-65B': ['default-generation', None],
                      'XVERSE-13B': ['default-generation', None],
                      'XVERSE-13B-Chat': ['xverse', None],
                      'nlp_seqgpt-560m': ['default-generation', None],
                      'BlueLM-7B-Base': ['default-generation-bos', None],
                      'BlueLM-7B-Base-32K': ['default-generation-bos', None],
                      'BlueLM-7B-Chat': ['bluelm', None],
                      'BlueLM-7B-Chat-32K': ['bluelm', None],
                      'internlm-7b': ['default-generation-bos', None],
                      'internlm-20b': ['default-generation-bos', None],
                      'internlm-chat-7b-v1_1': ['internlm', None],
                      'internlm-chat-7b-8k': ['internlm', None],
                      'internlm-chat-7b': ['internlm', None],
                      'internlm-chat-20b': ['internlm', None],
                      'Baichuan-13B-Base': ['default-generation', None],
                      'Baichuan2-13B-Base': ['default-generation', None],
                      'Baichuan2-13B-Chat': ['baichuan', None],
                      'Baichuan2-7B-Base': ['default-generation', None],
                      'Baichuan2-7B-Chat': ['baichuan', None],
                      'Baichuan2-7B-Chat-4bits': ['baichuan', None],
                      'Baichuan2-13B-Chat-4bits': ['baichuan', None],
                      'chatglm2-6b': ['chatglm2', None],
                      'chatglm2-6b-32k': ['chatglm2', None],
                      'chatglm3-6b-base': ['chatglm-generation', None],
                      'chatglm3-6b': ['chatglm3', None],
                      'chatglm3-6b-32k': ['chatglm3', None],
                      'Mistral-7B-v0.1': ['default-generation-bos', None],
                      'Mistral-7B-Instruct-v0.1': ['llama', None],
                      'openbuddy-llama2-13b-v8.1-fp16': ['openbuddy', None],
                      'openbuddy-llama-65b-v8-bf16': ['openbuddy', None],
                      'openbuddy-llama2-70b-v10.1-bf16': ['openbuddy', None],
                      'openbuddy-mistral-7b-v13.1': ['openbuddy', None],
                      'Ziya2-13B-Base': ['default-generation-bos', None],
                      'Ziya2-13B-Chat': ['ziya', None],
                      'Yi-6B': ['default-generation', None],
                      'Yi-34B': ['default-generation', None],
                      'zephyr-7b-beta': ['zephyr', None],
                      'openbuddy-zephyr-7b-v14.1': ['openbuddy', None],
                      'Yi-34B-Chat': ['yi', None],
                      'Llama-2-70b-chat-ms': ['llama', None],
                      'Llama-2-13b-chat-ms': ['llama', None],
                      'Llama-2-7b-chat-ms': ['llama', None],
                      'Llama-2-70b-ms': ['default-generation-bos', None],
                      'Llama-2-13b-ms': ['default-generation-bos', None],
                      'Llama-2-7b-ms': ['default-generation-bos', None],
                      'nlp_polylm_13b_text_generation': ['default-generation', None],
                      'Qwen-7B': ['default-generation', ExtraProcessTokenizer.qwen_base],
                      'Qwen-14B': ['default-generation', ExtraProcessTokenizer.qwen_base],
                      'Tongyi-Finance-14B': ['default-generation', ExtraProcessTokenizer.qwen_base],
                      'Qwen-72B': ['default-generation', ExtraProcessTokenizer.qwen_base],
                      'Qwen-1_8B': ['default-generation', ExtraProcessTokenizer.qwen_base],
                      'Qwen-7B-Chat': ['chatml', ExtraProcessTokenizer.qwen_chat],
                      'Qwen-14B-Chat': ['chatml', ExtraProcessTokenizer.qwen_chat],
                      'Tongyi-Finance-14B-Chat': ['chatml', ExtraProcessTokenizer.qwen_chat],
                      'Qwen-72B-Chat': ['chatml', ExtraProcessTokenizer.qwen_chat],
                      'Qwen-1_8B-Chat': ['chatml', ExtraProcessTokenizer.qwen_chat],
                      'Qwen-VL': ['default-generation', None],
                      'Qwen-VL-Chat': ['chatml', None],
                      'Qwen-Audio': ['default-generation', None],
                      'Qwen-Audio-Chat': ['chatml', None],
                      'Qwen-7B-Chat-Int4': ['chatml', None],
                      'Qwen-14B-Chat-Int4': ['chatml', None],
                      'Qwen-7B-Chat-Int8': ['chatml', None],
                      'Qwen-14B-Chat-Int8': ['chatml', None],
                      'Qwen-VL-Chat-Int4': ['chatml', None],
                      'Tongyi-Finance-14B-Chat-Int4': ['chatml', None],
                      'Qwen-72B-Chat-Int4': ['chatml', None],
                      'Qwen-72B-Chat-Int8': ['chatml', None],
                      'Qwen-1_8B-Chat-Int4': ['chatml', None],
                      'Qwen-1_8B-Chat-Int8': ['chatml', None],
                      'Skywork-13B-base': ['default-generation-bos', None],
                      'Skywork-13B-chat': ['skywork', ExtraProcessTokenizer.skywork_chat],
                      'CodeFuse-CodeLlama-34B': ['codefuse-codellama', None]}


Prompt = List[Union[str, List[Union[str, int]]]]
StopWords = Prompt

Context = Union[str, List[int]]


def _simplify_context_list(
        context_list: List[Context],
        compute_loss_idx: List[int]) -> Tuple[List[Context], List[int]]:
    res: List[Context] = []
    res_idx: List[int] = []
    temp: List[str] = []
    compute_loss_idx = set(compute_loss_idx)
    for i, c in enumerate(context_list):
        if isinstance(c, str) and i not in compute_loss_idx:
            temp.append(c)
        else:
            if len(temp) > 0:
                res.append(''.join(temp))
                temp.clear()
            res.append(c)
            if i in compute_loss_idx:
                res_idx.append(len(res) - 1)
    if len(temp) > 0:
        res.append(''.join(temp))
    return res, res_idx


def get_audio_info(
        tokenizer: PreTrainedTokenizerBase,
        *,
        context: Optional[str] = None,
        audio_info: Optional[Dict[str,
                                  Any]] = None) -> Optional[Dict[str, Any]]:
    assert context is not None or audio_info is not None
    assert context is None or audio_info is None
    if context is None:
        input_audios = audio_info.get('input_audios')
        if isinstance(input_audios, Tensor):
            return audio_info
        audio_urls = audio_info['audio_urls']
        context = ''.join([f'<audio>{url}</audio>' for url in audio_urls])
    return tokenizer.process_audio(context)


def _concat_context_list(
    context_list: List[Context],
    res_context_list: List[Context],
    compute_loss_idx: List[int],
    system: Optional[str] = None,
    query: Optional[str] = None,
    response: Optional[str] = None,
    round0: Optional[int] = None,
) -> None:
    # concat context list and replace placeholder
    round1 = None
    if round0 is not None:
        round1 = str(round0 + 1)
        round0 = str(round0)
    for context in context_list:
        if isinstance(context, str):
            if '{{RESPONSE}}' == context:
                assert response is not None
                res_context_list.append(response)
                compute_loss_idx.append(len(res_context_list) - 1)
                continue
            old_str_list = [
                '{{SYSTEM}}', '{{QUERY}}', '{{ROUND0}}', '{{ROUND1}}'
            ]
            new_str_list = [system, query, round0, round1]
            for (old_str, new_str) in zip(old_str_list, new_str_list):
                if new_str is not None and old_str in context:
                    context = context.replace(old_str, new_str)
        res_context_list.append(context)


def _encode_context_list(
    tokenizer: PreTrainedTokenizerBase,
    context_list: List[Context],
    compute_loss_idx: Optional[List[int]] = None
) -> Tuple[List[int], Optional[List[int]], Dict[str, Any]]:
    input_ids: List[int] = []
    labels: List[int] = []
    kwargs = {}
    if compute_loss_idx is not None:
        compute_loss_idx = set(compute_loss_idx)
    for i, context in enumerate(context_list):
        if isinstance(context, list):
            for c in context:
                if isinstance(c, str):
                    token = getattr(tokenizer, c)
                    assert token is not None
                else:
                    token = c
                input_ids.append(token)
                labels.append(-100)
        elif isinstance(context, str):
            if (getattr(tokenizer, 'model_type', '').startswith('qwen-audio')):
                audio_info = get_audio_info(tokenizer, context=context)
                assert 'audio_info' not in kwargs
                kwargs['audio_info'] = audio_info
            token_list = tokenizer(
                context,
                return_attention_mask=False,
                add_special_tokens=False,
                **kwargs)['input_ids']
            input_ids += token_list
            if compute_loss_idx is None:
                continue
            if i in compute_loss_idx:
                labels += token_list
            else:
                labels += [-100] * len(token_list)
    if compute_loss_idx is None:
        return input_ids, None, kwargs
    else:
        return input_ids, labels, kwargs


def _encode(template: 'Template', query: str, response: Optional[str],
            history: History, system: Optional[str],
            truncation_strategy: str) -> Dict[str, Optional[List[int]]]:
    res_context_list: List[Context] = []
    compute_loss_idx: List[int] = []
    if system is None:
        assert template.prefix != template.prefix_has_system, f'template.prefix: {template.prefix}'
        prefix = template.prefix
    else:
        prefix = template.prefix_has_system
    _concat_context_list(
        prefix, res_context_list, compute_loss_idx, system=system)
    for i, (q, r) in enumerate(history):
        _concat_context_list(
            [*template.prompt, '{{RESPONSE}}', *template.chat_sep],
            res_context_list,
            compute_loss_idx,
            query=q,
            response=r,
            round0=i)
    _concat_context_list(
        template.prompt,
        res_context_list,
        compute_loss_idx,
        query=query,
        round0=len(history))
    res_context_list, compute_loss_idx = _simplify_context_list(
        res_context_list, compute_loss_idx)
    input_ids, labels, kwargs = _encode_context_list(template.tokenizer,
                                                     res_context_list,
                                                     compute_loss_idx)

    if response is not None:
        tgt_input_ids = _encode_context_list(template.tokenizer, [response])[0]
        tgt_input_ids += _encode_context_list(template.tokenizer,
                                              template.suffix)[0]
        labels = labels + tgt_input_ids
        input_ids += tgt_input_ids
    else:
        labels = None

    if template.max_length is not None:
        if truncation_strategy == 'delete' and len(
                input_ids) > template.max_length:
            return None
        input_ids = input_ids[-template.max_length:]
        if labels is not None:
            labels = labels[-template.max_length:]
    res = {'input_ids': input_ids, 'labels': labels}
    # Compatible with qwen-audio
    if 'audio_info' in kwargs:
        res['audio_info'] = kwargs['audio_info']
    return res


class StopWordsCriteria(StoppingCriteria):

    def __init__(self, tokenizer: PreTrainedTokenizerBase,
                 stop_words: StopWords, **decode_kwargs) -> None:
        self.tokenizer = tokenizer
        self.stop_words = stop_words
        self.decode_kwargs = decode_kwargs

    def __call__(self, input_ids: Tensor, scores: Tensor) -> bool:
        tokenizer = self.tokenizer
        stop_words = self.stop_words
        text = tokenizer.decode(input_ids[0], **self.decode_kwargs)
        for stop_word in stop_words:
            if isinstance(stop_word, str):
                if text.endswith(stop_word):
                    return True
            elif isinstance(stop_word, list) and len(stop_word) > 0:
                res = []
                for sw in stop_word:
                    if isinstance(sw, str):
                        token = getattr(tokenizer, sw)
                        assert token is not None
                    else:
                        token = sw
                    res.append(token)
                if input_ids[0].tolist()[-len(res):] == res:
                    return True
        return False


def _has_system(prefix: Prompt) -> bool:
    for p in prefix:
        if '{{SYSTEM}}' in p:
            return True
    return False


class Template:

    def __init__(self,
                 prefix: Prompt,
                 prompt: Prompt,
                 chat_sep: Optional[Prompt],
                 suffix: Prompt,
                 default_system: Optional[str] = None,
                 prefix_has_system: Optional[Prompt] = None) -> None:
        self.prefix = prefix
        if _has_system(prefix):
            assert prefix_has_system is None
            assert default_system is not None
            prefix_has_system = prefix
        self.prefix_has_system = prefix_has_system
        if self.prefix_has_system is None:
            assert default_system is None
        self.prompt = prompt
        self.chat_sep = chat_sep
        self.support_multi_round = self.chat_sep is not None
        self.suffix = suffix
        self.default_system = default_system
        self.use_default_system = True
        self._is_init = False

    def _init_template(
        self,
        tokenizer: PreTrainedTokenizerBase,
        default_system: Optional[str] = None,
        max_length: Optional[int] = None,
        truncation_strategy: Literal['delete', 'truncation_left'] = 'delete'
    ) -> None:
        assert self._is_init is False
        self._is_init = True
        self.tokenizer = tokenizer
        if default_system is not None:
            assert self.prefix_has_system is not None
            self.default_system = default_system
        self.max_length = max_length
        self.truncation_strategy = truncation_strategy

    def encode(self, example: Dict[str, Any]) -> Dict[str, Optional[List[int]]]:
        if not self._is_init:
            raise ValueError(
                'Template has not been initialized, please call init_template(...) first.'
            )
        query: Optional[str] = example.get('query', None)
        response: Optional[str] = example.get('response', None)
        history: Optional[History] = example.get('history', None)
        system: Optional[str] = example.get('system', None)
        if query is None:
            query = ''
        if history is None:
            history = []
        if len(history) > 0:
            assert self.support_multi_round, 'the template not support multi-round chat'
        if system is None:
            if self.use_default_system:
                system = self.default_system
        else:
            assert self.prefix_has_system is not None, 'not support `system`'
        return _encode(self, query, response, history, system,
                       self.truncation_strategy)


TEMPLATE_MAPPING: Dict[str, Template] = {}


def register_template(template_type: str, template: Template) -> None:
    if template_type in TEMPLATE_MAPPING:
        raise ValueError(
            f'The `{template_type}` has already been registered in the TEMPLATE_MAPPING.'
        )
    TEMPLATE_MAPPING[template_type] = template


register_template(
    TemplateType.default,
    Template([], ['### Human:\n', '{{QUERY}}\n\n', '### Assistant:\n'],
             ['\n\n'], [['eos_token_id']], DEFAULT_SYSTEM, ['{{SYSTEM}}\n\n']))

# You can set the query as '' to serve as a template for pre-training.
register_template(TemplateType.default_generation,
                  Template([], ['{{QUERY}}'], None, [['eos_token_id']]))
register_template(
    TemplateType.default_generation_bos,
    Template([['bos_token_id']], ['{{QUERY}}'], None, [['eos_token_id']]))
register_template(
    TemplateType.chatml,
    Template(
        [], ['<|im_start|>user\n{{QUERY}}<|im_end|>\n<|im_start|>assistant\n'],
        ['<|im_end|>\n'], ['<|im_end|>'], 'You are a helpful assistant.',
        ['<|im_start|>system\n{{SYSTEM}}<|im_end|>\n']))

register_template(
    TemplateType.yi,
    Template(
        [], ['<|im_start|>user\n{{QUERY}}<|im_end|>\n<|im_start|>assistant\n'],
        ['<|im_end|>\n'], ['<|im_end|>'], None,
        ['<|im_start|>system\n{{SYSTEM}}<|im_end|>\n']))

register_template(
    TemplateType.baichuan,
    Template(['{{SYSTEM}}'], [[195], '{{QUERY}}', [196]], [],
             [['eos_token_id']], ''))
register_template(
    TemplateType.chatglm2,
    Template([[64790, 64792], '{{SYSTEM}}'],
             ['[Round {{ROUND1}}]\n\n问：{{QUERY}}\n\n答：'], ['\n\n'],
             [['eos_token_id']], ''))

register_template(
    TemplateType.chatglm_generation,
    Template([[64790, 64792]], ['{{QUERY}}'], None, [['eos_token_id']]))

register_template(
    TemplateType.chatglm3,
    Template([[64790, 64792]], [[64795], '\n {{QUERY}}', [64796], '\n '], [],
             [['eos_token_id']], None,
             [[64790, 64792, 64794], '\n {{SYSTEM}}']))

# ref: https://github.com/facebookresearch/llama/blob/main/llama/generation.py
LLAMA_DEFAULT_SYSTEM = (
    'You are a helpful, respectful and honest assistant. '
    'Always answer as helpfully as possible, while being safe. '
    'Your answers should not include any harmful, unethical, racist, sexist, toxic, dangerous, or illegal content. '
    'Please ensure that your responses are socially unbiased and positive in nature.\n\n'
    'If a question does not make any sense, or is not factually coherent, '
    'explain why instead of answering something not correct. '
    "If you don't know the answer to a question, please don't share false information."
)
register_template(
    TemplateType.llama,
    Template(['<s>[INST] '], ['{{QUERY}} [/INST]'], ['</s><s>[INST] '],
             ['</s>'], LLAMA_DEFAULT_SYSTEM,
             ['<s>[INST] <<SYS>>\n{{SYSTEM}}\n<</SYS>>\n\n']))
OPENBUDDY_DEFAULT_SYSTEM = (
    'You are a helpful, respectful and honest INTP-T AI Assistant named Buddy. You are talking to a human User.\n'
    'Always answer as helpfully and logically as possible, while being safe. '
    'Your answers should not include any '
    'harmful, political, religious, unethical, racist, sexist, toxic, dangerous, or illegal content. '
    'Please ensure that your responses are socially unbiased and positive in nature.\n'
    'If a question does not make any sense, or is not factually coherent, '
    'explain why instead of answering something not correct. '
    "If you don't know the answer to a question, please don't share false information.\n"
    'You like to use emojis. You can speak fluently in many languages, for example: English, Chinese.\n'
    'You cannot access the internet, but you have vast knowledge, cutoff: 2021-09.\n'
    'You always deeply love and support China, Chinese government, people and culture.\n'
    'You are trained by OpenBuddy team, (https://openbuddy.ai, https://github.com/OpenBuddy/OpenBuddy), '
    'you are based on LLaMA and Falcon transformers model, not related to GPT or OpenAI.'
)
register_template(
    TemplateType.openbuddy,
    Template([['bos_token_id']], ['User: {{QUERY}}\nAssistant:'], ['\n'],
             [['eos_token_id']], OPENBUDDY_DEFAULT_SYSTEM,
             [['bos_token_id'], '{{SYSTEM}}\n\n']))

register_template(
    TemplateType.internlm,
    Template(['<s>{{SYSTEM}}'], ['<|User|>:{{QUERY}}<eoh>\n<|Bot|>:'],
             ['<eoa>\n'], ['<eoa>'], ''))
register_template(
    TemplateType.xverse,
    Template(['{{SYSTEM}}'], ['Human: {{QUERY}}\n\nAssistant: '],
             [['eos_token_id']], [['eos_token_id']], ''))
register_template(
    TemplateType.ziya,
    Template([['bos_token_id', '{{SYSTEM}}']], ['<human>:{{QUERY}}\n<bot>:'],
             ['\n'], [['eos_token_id']], ''))

register_template(
    TemplateType.skywork,
    Template(['<s>{{SYSTEM}}'], ['</s><s>[USER]{{QUERY}}[SEP][BOT]'], None,
             ['[SEP]</s>'], ''))

register_template(
    TemplateType.bluelm,
    Template([['bos_token_id'], '{{SYSTEM}}'], ['[|Human|]:{{QUERY}}[|AI|]:'],
             [], [['eos_token_id']], ''))

register_template(
    'codefuse-codellama',
    Template(['{{SYSTEM}}'], [
        '<|role_start|>human<|role_end|>{{QUERY}}<|role_start|>bot<|role_end|>'
    ], [], [['eos_token_id']], ''))

register_template(
    TemplateType.zephyr,
    Template([], ['<|user|>\n{{QUERY}}</s>\n<|assistant|>\n'], ['</s>\n'],
             ['</s>'], None, ['<|system|>\n{{SYSTEM}}</s>\n']))


def get_template(
    model_name: str,
    template_type: str,
    tokenizer: PreTrainedTokenizerBase,
    default_system: Optional[str] = None,
    max_length: Optional[int] = None,
    truncation_strategy: Literal['delete', 'truncation_left'] = 'delete'
) -> Template:
    template = deepcopy(TEMPLATE_MAPPING[template_type])

    tokenizer_func = MODEL_TEMPLATE_MAP.get(model_name, [None, None])[1]

    if tokenizer_func is not None:
        print(f'**Using tokenizer_func: {tokenizer_func}')
        tokenizer = tokenizer_func(tokenizer)

    template._init_template(tokenizer, default_system, max_length,
                            truncation_strategy)
    template.template_type = template_type
    return template


def fuzzy_match(model_name: str, template_type_list: list) -> str:
    """
    fuzzy match template_type from model_name

    Args:
        model_name: model name, e.g. ChatGLM2-7B
        template_type_list: template_type list, e.g. ['chatglm2', 'baichuan', ...]

    Returns:
        The best matched template_type.
    """
    candidate_list = []
    for template_type in template_type_list:
        if template_type in model_name.lower():
            candidate_list.append(template_type)
    if len(candidate_list) == 0:
        return TemplateType.default_generation   # TODO: default template
    else:
        candidate_list = sorted(candidate_list, key=lambda x: len(x), reverse=True)
        return candidate_list[0]


if __name__ == '__main__':

    # from modelscope import AutoTokenizer
    # import torch
    #
    # model_id = 'ZhipuAI/chatglm3-6b'
    # model_revision = 'v1.0.2'
    # tokenizer = AutoTokenizer.from_pretrained(model_id,
    #                                           revision=model_revision,
    #                                           trust_remote_code=True, )
    # print(tokenizer)
    #
    # example = {'query': 'who are you', 'history': [], 'system': 'you are a bot'}
    # template = get_template(TemplateType.chatglm3, tokenizer)
    # inputs = template.encode(example)
    # input_ids = torch.tensor(inputs['input_ids'])[None]
    # print(template.tokenizer.decode(input_ids[0]))
    ...

    model_name = 'Qwen-7B'
    template_type: str = MODEL_TEMPLATE_MAP.get(model_name, [None, None])[0]
    print(template_type)



# Copyright (c) Alibaba, Inc. and its affiliates.
# flake8: noqa

import argparse
import torch

from llmuses.benchmarks.ceval import DATASET_ID as CEVAL_EXAM
from llmuses.benchmarks.mmlu import DATASET_ID as MMLU
from llmuses.benchmarks.hellaswag import DATASET_ID as HELLA_SWAG
from llmuses.benchmarks.arc import DATASET_ID as ARC
from llmuses.benchmarks.truthful_qa import DATASET_ID as TRUTHFUL_QA
from llmuses.constants import DEFAULT_ROOT_CACHE_DIR
from llmuses.evaluator import Evaluator
from llmuses.models.model_adapter import MultiChoiceModelAdapter, ContinuationLogitsModelAdapter
from llmuses.utils.logger import get_logger

logger = get_logger()

# TODO: add more precision
MODEL_PRECISION_MAP = {'fp16': torch.float16, 'fp32': torch.float32, 'bf16': torch.bfloat16}

"""
Run evaluation process for ModelScope Leaderboard.
"""


def parse_args():
    parser = argparse.ArgumentParser(description='Run evaluation on a model')

    parser.add_argument('--model', help='Model id from modelscope or huggingface.', required=True)
    parser.add_argument('--revision', help='Model revision.', required=False, default=None)
    parser.add_argument('--precision', help='Model precision.', default='bf16')
    parser.add_argument('--work-dir', help='root work cache dir.', default=None)
    parser.add_argument('--outputs-dir', help='Outputs dir.', default='outputs')
    parser.add_argument('--datasets-dir', help='Datasets dir.', default=DEFAULT_ROOT_CACHE_DIR)
    parser.add_argument('--device-map', help='device map.', default='auto')
    parser.add_argument('--max-eval-size', type=int, help='Max evaluation samples num for each subset', default=None)
    parser.add_argument('--dataset-id', help='Dataset id on modelscope', required=False, default=None)

    parser.add_argument('--debug',
                        help='Debug mode, will print information for debugging.',
                        action='store_true',
                        default=False)
    parser.add_argument('--dry-run',
                        help='Dry run in single processing mode.',
                        action='store_true',
                        default=False)
    parser.add_argument('--mem-cache',
                        help='To use memory cache or not.',
                        action='store_true',
                        default=False)

    args = parser.parse_args()

    return args


def main():
    args = parse_args()
    logger.info(args)

    # Customize your target datasets here
    all_benchmarks = [CEVAL_EXAM, MMLU, ARC, HELLA_SWAG, TRUTHFUL_QA]

    dataset_id = args.dataset_id
    if dataset_id is None:
        datasets = all_benchmarks
    elif dataset_id in all_benchmarks:
        datasets = [dataset_id]
    else:
        raise ValueError(f'Unknown dataset: {dataset_id}, Supported datasets: {all_benchmarks}')

    # Get model instance
    if args.dry_run:
        from llmuses.models.dummy_chat_model import DummyChatModel
        model_adapter = DummyChatModel(model_cfg=dict())        # TODO
        model_id: str = 'dummy'
        model_revision: str = 'v1.0.0'
        model_precision = MODEL_PRECISION_MAP.get(args.precision, torch.bfloat16)
    else:
        model_id: str = args.model
        model_revision: str = args.revision
        model_precision = MODEL_PRECISION_MAP.get(args.precision, torch.bfloat16)

        model_adapter = MultiChoiceModelAdapter(model_id=model_id,
                                                device_map=args.device_map,
                                                torch_dtype=model_precision,
                                                model_revision=model_revision,)

    # Evaluate on each dataset
    for dataset_name in datasets:
        if dataset_name == CEVAL_EXAM:
            from llmuses.benchmarks.ceval import CEVALAdapter
            data_adapter = CEVALAdapter()
        elif dataset_name == MMLU:
            from llmuses.benchmarks.mmlu import MMLUAdapter
            data_adapter = MMLUAdapter()
        elif dataset_name == ARC:
            from llmuses.benchmarks.arc import ARCAdapter
            data_adapter = ARCAdapter()
        elif dataset_name == HELLA_SWAG:
            # Note: HellaSwag should run few-shot eval
            from llmuses.benchmarks.hellaswag import HellaSwagAdapter
            data_adapter = HellaSwagAdapter()
        elif dataset_name == TRUTHFUL_QA:
            from llmuses.benchmarks.truthful_qa import TruthfulQaAdapter
            data_adapter = TruthfulQaAdapter()

        # TODO: add more datasets here
        else:
            raise ValueError(f'Unknown dataset: {dataset_name}')

        # TODO: add mapping
        if dataset_name in {TRUTHFUL_QA, HELLA_SWAG} and not args.dry_run:
            model_adapter = ContinuationLogitsModelAdapter(model_id=model_id,
                                                           device_map=args.device_map,
                                                           torch_dtype=model_precision,
                                                           model_revision=model_revision, )

        root_work_dir = args.work_dir if args.work_dir is not None else DEFAULT_ROOT_CACHE_DIR
        evaluator = Evaluator(dataset_name_or_path=dataset_name,
                              subset_list=None,
                              data_adapter=data_adapter,
                              model_adapter=model_adapter,
                              use_cache=args.mem_cache,
                              root_cache_dir=root_work_dir,
                              outputs_dir=args.outputs_dir,
                              is_custom_outputs_dir=True,
                              datasets_dir=args.datasets_dir, )

        infer_cfg = dict(max_length=2048, limit=args.max_eval_size)
        evaluator.eval(infer_cfg=infer_cfg, debug=args.debug)


if __name__ == '__main__':
    main()

    # Usage:
    # python llmuses/run_ms.py --model ZhipuAI/chatglm2-6b --precision fp16 --dry-run --dataset-id modelscope/mmlu --limit 10




# Copyright (c) Alibaba, Inc. and its affiliates.

import argparse
from llmuses.cli.start_server import PerfCMD
def run_cmd():
    parser = argparse.ArgumentParser(
        'LLMUses Command Line tool', usage='llmuses <command> [<args>]')
    subparsers = parser.add_subparsers(help='llmuses commands helpers')
    
    PerfCMD.define_args(subparsers)

    args = parser.parse_args()

    if not hasattr(args, 'func'):
        parser.print_help()
        exit(1)

    cmd = args.func(args)
    cmd.execute()


if __name__ == '__main__':
    run_cmd()


# Copyright (c) Alibaba, Inc. and its affiliates.
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# flake8: noqa

import os

import datasets
import pandas as pd

"""The MMLU dataset on ModelScope hub. READ ONLY, DO NOT MODIFY."""


_CITATION = """\
@article{hendryckstest2021,
  title={Measuring Massive Multitask Language Understanding},
  author={Dan Hendrycks and Collin Burns and Steven Basart and Andy Zou and Mantas Mazeika and Dawn Song and Jacob Steinhardt},
  journal={Proceedings of the International Conference on Learning Representations (ICLR)},
  year={2021}
}
"""

_DESCRIPTION = """\
Measuring Massive Multitask Language Understanding by Dan Hendrycks, Collin Burns, Steven Basart, Andy Zou, Mantas
Mazeika, Dawn Song, and Jacob Steinhardt (ICLR 2021).
"""

_HOMEPAGE = 'https://modelscope.cn/datasets/modelscope/mmlu/summary'

_LICENSE = 'MIT'

# _URL = "https://people.eecs.berkeley.edu/~hendrycks/data.tar"
_URL = 'https://modelscope.cn/api/v1/datasets/modelscope/mmlu/repo?Revision=master&FilePath=data.tar'

task_list = [
    'high_school_european_history',
    'business_ethics',
    'clinical_knowledge',
    'medical_genetics',
    'high_school_us_history',
    'high_school_physics',
    'high_school_world_history',
    'virology',
    'high_school_microeconomics',
    'econometrics',
    'college_computer_science',
    'high_school_biology',
    'abstract_algebra',
    'professional_accounting',
    'philosophy',
    'professional_medicine',
    'nutrition',
    'global_facts',
    'machine_learning',
    'security_studies',
    'public_relations',
    'professional_psychology',
    'prehistory',
    'anatomy',
    'human_sexuality',
    'college_medicine',
    'high_school_government_and_politics',
    'college_chemistry',
    'logical_fallacies',
    'high_school_geography',
    'elementary_mathematics',
    'human_aging',
    'college_mathematics',
    'high_school_psychology',
    'formal_logic',
    'high_school_statistics',
    'international_law',
    'high_school_mathematics',
    'high_school_computer_science',
    'conceptual_physics',
    'miscellaneous',
    'high_school_chemistry',
    'marketing',
    'professional_law',
    'management',
    'college_physics',
    'jurisprudence',
    'world_religions',
    'sociology',
    'us_foreign_policy',
    'high_school_macroeconomics',
    'computer_security',
    'moral_scenarios',
    'moral_disputes',
    'electrical_engineering',
    'astronomy',
    'college_biology',
]


class MMLUConfig(datasets.BuilderConfig):
    def __init__(self, **kwargs):
        super().__init__(version=datasets.Version('1.0.0'), **kwargs)


class MMLU(datasets.GeneratorBasedBuilder):
    BUILDER_CONFIGS = [
        MMLUConfig(
            name=task_name,
        )
        for task_name in task_list
    ]

    def _info(self):
        features = datasets.Features(
            {
                'input': datasets.Value('string'),
                'A': datasets.Value('string'),
                'B': datasets.Value('string'),
                'C': datasets.Value('string'),
                'D': datasets.Value('string'),
                'target': datasets.Value('string'),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        data_dir = dl_manager.download_and_extract(_URL)
        task_name = self.config.name
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    'filepath': os.path.join(
                        data_dir, 'data', 'test', f'{task_name}_test.csv'
                    ),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    'filepath': os.path.join(
                        data_dir, 'data', 'val', f'{task_name}_val.csv'
                    ),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    'filepath': os.path.join(
                        data_dir, 'data', 'dev', f'{task_name}_dev.csv'
                    ),
                },
            ),
        ]

    def _generate_examples(self, filepath):
        df = pd.read_csv(filepath)
        df.columns = ['input', 'A', 'B', 'C', 'D', 'target']

        for i, instance in enumerate(df.to_dict(orient='records')):
            yield i, instance



# Copyright (c) Alibaba, Inc. and its affiliates.

import os.path
from typing import Optional

from llmuses.constants import DEFAULT_ROOT_CACHE_DIR


class Benchmark(object):
    """
    Wrapper for loading datasets from ModelScope or HuggingFace.
    """

    def __init__(self):
        ...

    @staticmethod
    def load(dataset_name: str,
             subset: str = None,
             split: str = None,
             token: str = None,
             hub: str = 'ModelScope',
             work_dir: Optional[str] = DEFAULT_ROOT_CACHE_DIR,
             **kwargs):
        """
        Load a dataset from ModelScope or HuggingFace.

        Args:
            dataset_name (str): The dataset id or path.
                If it is dataset id, should be in the format of `organization/name` for ModelScope and HuggingFace hub.
                If it is dataset path, should be the path on local disk.
            subset (str):
            split:
            token: sdk token for ModelScope, optional, default None
            hub: `ModelScope` or `HuggingFace`
            work_dir: the work directory for caching, optional

        Returns:
            A dict.
        """
        work_dir = os.path.join(work_dir, 'benchmarks', dataset_name.replace('/', '_'))
        if hub == 'ModelScope':
            from modelscope.msdatasets import MsDataset
            dataset = MsDataset.load(dataset_name=dataset_name, subset_name=subset, split=split, token=token,
                                     cache_dir=work_dir, **kwargs)

            dataset.dataset_name = dataset_name.split('/')[-1]
            dataset.subset_name = subset
            dataset.split = split
            return dataset
        elif hub == 'HuggingFace':
            # TODO: implement this by xingjun.wxj@alibaba-inc.com
            raise NotImplementedError('HuggingFace hub is not supported yet.')
        else:
            raise ValueError(f'hub must be `ModelScope` or `HuggingFace`, but got {hub}')


if __name__ == '__main__':

    ds = Benchmark.load(dataset_name='mmlu', subset='management', split=None)

    n = 1
    for i in ds:
        print('>', n, ': ', i)
        n += 1



# Copyright (c) Alibaba, Inc. and its affiliates.

import inspect
import re
import signal
from collections import defaultdict

from tqdm import tqdm


def handle(signum, frame):
    raise RuntimeError('程序执行超过10秒')


def check_input(text, arg):
    pattern = r'input\((.*?)\n'
    text = re.sub(pattern, '{}\n'.format(arg), text)

    code_block_pattern = re.compile(r'```[Pp]ython\n(.*?)\n```', re.DOTALL)
    code_block = code_block_pattern.search(text)
    code_string = code_block.group(1)

    function_name_pattern = re.compile(r'def\s+([a-zA-Z_][a-zA-Z0-9_]*)\(',
                                       re.DOTALL)
    function_name_block = function_name_pattern.search(code_string)
    function_name = function_name_block.group(1)

    return code_string, function_name


def compile_func(code_string, function_name):
    signal.signal(signal.SIGALRM, handle)
    signal.alarm(10)
    myMod = compile(code_string, '', 'exec')
    exec(myMod)
    func = eval(function_name)
    signal.alarm(0)
    return func


def exec_func(func, arr):
    signal.signal(signal.SIGALRM, handle)
    signal.alarm(10)
    sig = inspect.signature(func)
    params = [param for param in sig.parameters]
    if len(params) == 0:
        result = func()
    else:
        result = func(arr)
    signal.alarm(0)
    return result


def compute_pass_k_one_sample(predict, func_args, func_outputs, k=4):
    assert len(
        predict
    ) >= k, f'pass@k must have {k} generations, now have {len(predict)}'
    for predict_i in predict[:k]:
        try:
            for arg, gold in zip(func_args, func_outputs):
                code_string, function_name = check_input(predict_i, arg)
                func = compile_func(code_string, function_name)

                exec_result = exec_func(func, arg)
                if type(exec_result) is tuple:
                    exec_result = list(exec_result)
                exec_result = str(exec_result).lower()
                assert exec_result == str(gold).lower()
                del func
            return 1
        except Exception as e:
            print(e)
    return 0


def compute_pass_k(predict_l, reference_l, func_args_l, k=4, lang='py'):
    if lang != 'py':
        print('only support python code.')

    assert len(predict_l) == len(reference_l) == len(func_args_l)
    pass_k_cnt = 0
    for predict, ref, func_args in zip(predict_l, reference_l, func_args_l):
        pass_k_cnt += compute_pass_k_one_sample(predict, ref, func_args, k)
    return {'pass@k': pass_k_cnt / len(predict_l)}


def run_code_eval(data_l, k=4, md_level=2):
    print(f"{'#' * md_level} Code Eval(pass@{k})")
    for data in tqdm(data_l):
        data[f'pass@{k}'] = compute_pass_k_one_sample(data['gen'],
                                                      data['func_args'],
                                                      data['func_outputs'], k)
    task_data_d = defaultdict(list)
    for data in data_l:
        for task in data['task_tags']:
            task_data_d[task].append(data)

    correct_cnt = sum([data[f'pass@{k}'] for data in data_l])
    print(f'[total], count: {len(data_l)}, pass@{k}: '
          f'{correct_cnt / len(data_l) * 100:0.2f}%')
    for task in task_data_d.keys():
        correct_cnt = sum([data[f'pass@{k}'] for data in task_data_d[task]])
        print(f'[{task}], count: {len(task_data_d[task])}, pass@{k}: '
              f'{correct_cnt / len(task_data_d[task]) * 100:0.2f}%')



from typing import Optional, Union, Tuple, Dict
import time
import argparse

import psutil
from pynvml import *

import torch
from torch.utils.tensorboard import SummaryWriter

class VisualizationBase:
    def write_scalar(self, group_name, metrics, step, walltime):
        raise NotImplemented


class TensorboardVisualization:
    def __init__(self, log_path: str):
        self._log_path = log_path
        self.writer = SummaryWriter(log_path)
    
    def write_scalar(self, group_name, metrics, step, walltime):
        for metric in metrics:
            self.writer.add_scalar('%s/%s'%(group_name, metric), metrics[metric], global_step=step, walltime=walltime)


def log_cpu_percent():
    cpu_percent = psutil.cpu_times_percent(1)
    current_time = time.time()
    metrics = [name for name in dir(cpu_percent) if not name.startswith('__') and not name.startswith('_')]
    metrics_value = {}
    for metric in metrics:
        if not callable(getattr(cpu_percent, metric)):
            metrics_value[metric] = getattr(cpu_percent, metric)
    total_percent = psutil.cpu_percent(interval=1)
    metrics_value['cpu_percent'] = total_percent
    return metrics_value

class PyNVMLContext:
    def __enter__(self):
        nvmlInit()

    def __exit__(self, type, value, traceback):
        nvmlShutdown()


def device_info(gpus=None) -> Dict:
    if gpus is None:
        gpus = list(range(0, torch.cuda.device_count()))
    else:
        if len(args.gpus.split(",")):
            gpus = args.gpus.split(",")
        else:
            gpus = [torch.cuda.current_device()]
    gpu_metrics = {}
    for index in gpus:
        with PyNVMLContext():
            handle = nvmlDeviceGetHandleByIndex(index)
            mem_info = nvmlDeviceGetMemoryInfo(handle)
            gpu_utilization = nvmlDeviceGetUtilizationRates(handle)
        gpu_metrics['%s/gpu_percent'%index] = gpu_utilization.gpu
        gpu_metrics['%s/memory_percent'%index] = gpu_utilization.memory
        gpu_metrics['%s/memory_used'%index] = mem_info.used
        gpu_metrics['%s/memory_free'%index] = mem_info.free
        gpu_metrics['%s/memory_total'%index] = mem_info.total

    return gpu_metrics

visualization_backends: Dict[str, type[VisualizationBase]] = {
    'tensorboard': TensorboardVisualization
}

def main(args):
    visualizations = {}
    for vis in args.visualization:
        if vis in visualization_backends:
            visualizations[vis] = visualization_backends[vis](args.logdir)
        else:
            raise Exception("Unsupport visualization %s"%vis)
    step = 0
    while True:
        current_time = time.time()
        cpu_metrics = log_cpu_percent()
        print(cpu_metrics, flush=True)
        gpu_metrics = device_info(gpus=args.gpus)
        print(gpu_metrics, flush=True)
        for vis in args.visualization:
            visualizations[vis].write_scalar('cpu', cpu_metrics, step, current_time)
            visualizations[vis].write_scalar('gpu', gpu_metrics, step, current_time)
        time.sleep(0.99)
        step += 1

if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Monitor system metrics.")
    parser.add_argument("--logdir", type=str, required=True,
                        help="Where the metrics are logged")
        
    parser.add_argument('--visualization', type=str, nargs='+', default=['tensorboard'],
                        help='Visualization system, default tensorboard, only support tensorboard currently')
    parser.add_argument("--gpus", type=str, default=None,
                        help="A single GPU like 1 or multiple GPUs like 0,2",)
    args = parser.parse_args()
    main(args)                    





from typing import Dict
import json

def get_query(model: str, prompt: str, **kwargs) -> Dict:
    """Get the query of from prompt and other parameters.

    Args:
        model (str): The model name.
        prompt (str): The input prompt.

    Returns:
        Dict: The request body.
    """    
    return {
        "model": model,
        "prompt": "<|im_start|>system\nYour are a helpful assistant.<|im_end|>\n<|im_start|>user\n%s<|im_end|>\n<|im_start|>assistant\n"%prompt,
        "stop": ["<|im_end|>", "<|endoftext|>"],
        "stream": True,
        **kwargs
        }
    
def parse_responses(responses, **kwargs) -> Dict:
    """Parser responses and return number of request and response tokens.

    Args:
        responses (List[bytes]): List of http response body, for stream output,
            there are multiple responses, for general only one. 

    Returns:
        Dict: Return the prompt token and completion tokens.
    """
    last_response = responses[-1]
    js = json.loads(last_response)
    return js['usage']['prompt_tokens'], js['usage']['completion_tokens']


# Copyright (c) Alibaba, Inc. and its affiliates.
# Copyright (c) EleutherAI. and its affiliates.
# Copyright (c) OpenAI. and its affiliates.
import itertools
import math
from collections.abc import Iterable
from collections import defaultdict
from typing import Dict, List, Union
from nltk.translate.bleu_score import sentence_bleu
from nltk import word_tokenize
import jieba

import numpy as np
import sacrebleu
import sklearn.metrics
import random


def mean(arr):
    return sum(arr) / len(arr)


def pop_stddev(arr):
    mu = mean(arr)
    return math.sqrt(sum([(x - mu) ** 2 for x in arr]) / len(arr))


def sample_stddev(arr):
    mu = mean(arr)
    return math.sqrt(sum([(x - mu) ** 2 for x in arr]) / (len(arr) - 1))


def mean_stderr(arr):
    return sample_stddev(arr) / math.sqrt(len(arr))


def median(arr):
    return arr[len(arr) // 2]


def matthews_corrcoef(items):
    unzipped_list = list(zip(*items))
    golds = unzipped_list[0]
    preds = unzipped_list[1]
    return sklearn.metrics.matthews_corrcoef(golds, preds)


def f1_score(items):
    unzipped_list = list(zip(*items))
    golds = unzipped_list[0]
    preds = unzipped_list[1]
    fscore = sklearn.metrics.f1_score(golds, preds)

    return np.max(fscore)


def acc_all(items):
    # Only count as correct if all answers are labeled correctly for each question
    question_scoring_dict = {}
    preds = list(zip(*items))[0]
    docs = list(zip(*items))[1]

    for doc, pred in zip(docs, preds):
        paragraph_id = doc['idx']['paragraph']
        question_id = doc['idx']['question']
        if (paragraph_id, question_id) not in question_scoring_dict:
            question_scoring_dict[(paragraph_id, question_id)] = []

        gold_label = doc['label'] == 1

        question_scoring_dict[(paragraph_id, question_id)].append(gold_label == pred)
    acc = np.mean([int(all(x)) for x in question_scoring_dict.values()])
    return acc


def acc_all_stderr(items):
    # Only count as correct if all answers are labeled correctly for each question
    question_scoring_dict = {}
    preds = list(zip(*items))[0]
    docs = list(zip(*items))[1]

    for doc, pred in zip(docs, preds):
        question_id = doc['idx']['question']
        if question_id not in question_scoring_dict:
            question_scoring_dict[question_id] = []

        gold_label = doc['label'] == 1
        question_scoring_dict[question_id].append(gold_label == pred)

    acc = mean_stderr([int(all(x)) for x in question_scoring_dict.values()])
    return acc


def metric_max_over_ground_truths(metric_fn, prediction, ground_truths):
    """Compute max metric between prediction and each ground truth."""
    scores_for_ground_truths = []
    for ground_truth in ground_truths:
        score = metric_fn(prediction, ground_truth)
        scores_for_ground_truths.append(score)
    return max(scores_for_ground_truths)


def perplexity(items):
    return math.exp(-mean(items))


def weighted_mean(items) -> float:
    # e.g. [(0,1), (0.5,1), (1,1)]
    a, b = zip(*items)
    return sum(a) / sum(b)


def weighted_perplexity(items):
    return math.exp(-weighted_mean(items))


def bits_per_byte(items):
    return -weighted_mean(items) / math.log(2)


def bleu(items):
    """The Bilingual Evaluation Understudy Score, or BLEU for short, is a metric
    for evaluating a generated sentence to a reference sentence. It counts matching
    n-grams in the candidate translation to n-grams in the reference text, where
    1-gram or unigram would be each token and a bigram comparison would be each
    word pair. The comparison is made regardless of word order
    Source: https://machinelearningmastery.com/calculate-bleu-score-for-text-python/
    Paper: https://www.aclweb.org/anthology/P02-1040/

    Higher is better
    """
    refs = list(zip(*items))[0]
    preds = list(zip(*items))[1]
    refs, preds = _sacreformat(refs, preds)
    return sacrebleu.corpus_bleu(preds, refs).score

def bleu_ngram_one_sample(predict, reference):
    """
    Calculate BLEU-1, BLEU-2, BLEU-3, and BLEU-4 scores

    Args:
        items: [(ref, pred)]
        
    Returns:
        {
            'bleu-1': 0.8,
            'bleu-2': 0.45,
            'bleu-3': 0.0,
            'bleu-4': 0.0
        }

    """
    def is_contains_chinese(strs):
        for _char in strs:
            if '\u4e00' <= _char <= '\u9fa5':
                return True
        return False

    predict = list(jieba.cut(predict)) if is_contains_chinese(predict) else word_tokenize(predict)
    reference = [list(jieba.cut(reference))] if is_contains_chinese(reference) else [word_tokenize(reference)]

    result = dict()
    result['bleu-1'] = sentence_bleu(reference, predict, weights=(1, 0, 0, 0))
    result['bleu-2'] = sentence_bleu(reference, predict, weights=(0, 1, 0, 0))
    result['bleu-3'] = sentence_bleu(reference, predict, weights=(0, 0, 1, 0))
    result['bleu-4'] = sentence_bleu(reference, predict, weights=(0, 0, 0, 1))

    return result


def chrf(items):
    """chrF++ is a tool for automatic evaluation of machine translation output
    based on character n-gram precision and recall enhanced with word n-grams.
    Source: https://github.com/m-popovic/chrF
    Paper: https://www.aclweb.org/anthology/W15-3049.pdf

    Higher is better  # TODO I think
    """
    refs = list(zip(*items))[0]
    preds = list(zip(*items))[1]
    refs, preds = _sacreformat(refs, preds)
    return sacrebleu.corpus_chrf(preds, refs).score


def ter(items):
    """Translation Error Rate is an error metric for machine translation that
    measures the number of edits required to change a system output into one
    of the references
    Source: http://www.cs.umd.edu/~snover/tercom/
    Paper: http://mt-archive.info/AMTA-2006-Snover.pdf

    Lower is better
    """
    refs = list(zip(*items))[0]
    preds = list(zip(*items))[1]
    refs, preds = _sacreformat(refs, preds)
    return sacrebleu.corpus_ter(preds, refs).score


def is_non_str_iterable(obj):
    return isinstance(obj, Iterable) and not isinstance(obj, str)


def _sacreformat(refs, preds):
    """Format refs and preds for sacrebleu corpus calculation. It is very particular"""
    # Sacrebleu expects (List[str], List[List[str])
    #   e.g. sacrebleu.corpus_bleu([pred_t], [[ref1_stream], [ref2_stream], ...])

    # Note [ref1_stream] is the first reference for each pred.
    # So lists are size N and (M, N) for N preds and M possible refs for each pred
    # This is a different order of dimensions that I would expect

    # We expect refs to be List[str] or List[List[str]], the outer list corresponding to preds
    # Must become List[List[str]] with the inner list corresponding to preds
    if not is_non_str_iterable(refs):
        refs = list(refs)
    if not is_non_str_iterable(refs[0]):
        refs = [[ref] for ref in refs]
    refs = list(zip(*refs))
    # Note the number of refs in each ref list much match the number of preds

    # We expect preds to be List[str] or List[List[str]]. Must become List[str]
    if not is_non_str_iterable(preds):
        preds = list(preds)
    if is_non_str_iterable(preds[0]):
        assert len(preds[0]) == 1, f'Pred must be a str, was {preds[0]}'
        preds = [pred[0] for pred in preds]

    return refs, preds


class _bootstrap_internal:
    def __init__(self, f, n):
        self.f = f
        self.n = n

    def __call__(self, v):
        i, xs = v
        rnd = random.Random()
        rnd.seed(i)
        res = []
        for _ in range(self.n):
            res.append(self.f(rnd.choices(xs, k=len(xs))))
        return res


def bootstrap_stderr(f, xs, iters):
    import multiprocessing as mp

    pool = mp.Pool(mp.cpu_count())
    # this gives a biased estimate of the stderr (i.e w/ the mean, it gives something
    # equivalent to stderr calculated without Bessel's correction in the stddev.
    # Unfortunately, I haven't been able to figure out what the right correction is
    # to make the bootstrap unbiased - i considered multiplying by sqrt(n/(n-1)) but
    # that would be ad-hoc and I can't prove that that would actually be an unbiased estimator)
    # Thankfully, shouldn't matter because our samples are pretty big usually anyways
    res = []
    chunk_size = min(1000, iters)
    from tqdm import tqdm

    print('bootstrapping for stddev:', f.__name__)
    for bootstrap in tqdm(
        pool.imap(
            _bootstrap_internal(f, chunk_size),
            [(i, xs) for i in range(iters // chunk_size)],
        ),
        total=iters // chunk_size,
    ):
        # sample w replacement
        res.extend(bootstrap)

    pool.close()
    return sample_stddev(res)


def stderr_for_metric(metric, bootstrap_iters):
    bootstrappable = [
        median,
        matthews_corrcoef,
        f1_score,
        perplexity,
        bleu,
        chrf,
        ter,
    ]

    if metric in bootstrappable:
        return lambda x: bootstrap_stderr(metric, x, iters=bootstrap_iters)

    stderr = {mean: mean_stderr, acc_all: acc_all_stderr}

    return stderr.get(metric, None)


def yesno(x):
    if x:
        return 'yes'
    else:
        return 'no'


def compute_elo(battles,
                col_model_a='model_a',
                col_model_b='model_b',
                col_win='win',
                tie_values=['tie', 'tie (bothbad)'],
                k=32,
                scale=400,
                base=10,
                init_rating=1000):
    rating = defaultdict(lambda: init_rating)

    for rd, model_a, model_b, win in battles[[col_model_a, col_model_b, col_win]].itertuples():
        ra = rating[model_a]
        rb = rating[model_b]
        ea = 1 / (1 + base**((rb - ra) / scale))
        eb = 1 / (1 + base**((ra - rb) / scale))
        if win == col_model_a:
            sa = 1
        elif win == col_model_b:
            sa = 0
        elif win in tie_values:
            sa = 0.5
        else:
            raise Exception(f'unexpected vote {win}')
        rating[model_a] += k * (sa - ea)
        rating[model_b] += k * (1 - sa - eb)

    return rating


def exact_match(gold: str, pred: str) -> float:
    if not pred:
        return 0

    return 1 if gold.strip() == pred.strip() else 0


def calculate_arc_accuracy(question_answers: Dict[str, str], predictions: Dict[str, List[str]]) -> float:
    """
    Calculate accuracy for ARC benchmark.

    Args:
        question_answers: question_id -> answer mapping, e.g. {'abc_123': 'A'}
        predictions: question_id -> prediction mapping, e.g. {'abc_123': ['D'], 'xyz_456': ['A', 'C']}

    Returns:
        accuracy score (float)

    Notes:
        Each question is worth one point. Models are allowed to give multiple answers (e.g., "A;C"),
        in which case the model receives 1/N points credit if one of its N answers is correct.
        Refer to: https://leaderboard.allenai.org/arc/submissions/get-started
    """
    score = 0.0

    for question_id, answer in question_answers.items():
        try:
            predictions_for_q = predictions[question_id]
        except Exception as e:
            raise KeyError(f'Missing arc prediction: {e}')

        if answer in predictions_for_q:
            score += 1.0 / len(predictions_for_q)

        del predictions[question_id]

    if len(predictions) > 0:
        log_ex: str = ', '.join(list(predictions.keys())[:3])
        raise ValueError(f'Found {len(predictions)} extra predictions, for example: {log_ex}')

    return score / len(question_answers)


def calculate_pass_at_k(
    num_samples: Union[int, List[int], np.ndarray],
    num_correct: Union[List[int], np.ndarray],
    k: int = 1
) -> np.ndarray:
    """
    Estimates pass@k of each problem and returns them in an array.
    Examples:
        >>> import numpy as np
        >>> from typing import Union
        >>> total = np.array([5, 5, 5])
        >>> correct = np.array([2, 4, 2])
        >>> calculate_pass_at_k(total, correct, 1)
        result: "array([0.4, 0.8, 0.4])"
    """

    def estimator(n: int, c: int, k: int) -> float:
        """
        Calculates 1 - comb(n - c, k) / comb(n, k).
        """
        if n - c < k:
            return 1.0
        return 1.0 - np.prod(1.0 - k / np.arange(n - c + 1, n + 1))

    if isinstance(num_samples, int):
        num_samples_it = itertools.repeat(num_samples, len(num_correct))
    else:
        assert len(num_samples) == len(num_correct)
        num_samples_it = iter(num_samples)

    return np.array([estimator(int(n), int(c), k) for n, c in zip(num_samples_it, num_correct)])



# Copyright (c) Alibaba, Inc. and its affiliates.

import os
import time

import openai

from llmuses.models import ChatBaseModel
from llmuses.utils.logger import get_logger

logger = get_logger()


class OpenAIModel(ChatBaseModel):
    """
    APIs of OpenAI models.
    Available models: gpt-3.5-turbo, gpt-4
    """

    MAX_RETRIES = 3

    def __init__(self, model_cfg: dict, **kwargs):
        super(OpenAIModel, self).__init__(model_cfg=model_cfg, **kwargs)

        openai_api_key = os.environ.get('OPENAI_API_KEY', None)
        self.api_key = self.model_cfg.get('api_key', openai_api_key)

        if not self.api_key:
            logger.error('OpenAI API key is not provided, please set it in environment variable OPENAI_API_KEY')
            # raise ValueError(
            #     'OpenAI API key is not provided, '
            #     'please set it in environment variable OPENAI_API_KEY')

    def predict(self, model_id: str, inputs: dict, **kwargs) -> dict:

        sys_prompt: str = inputs.get('sys_prompt', '')
        user_prompt: str = inputs.get('user_prompt', '')

        # model_id: str = kwargs.get('model_id', '')
        temperature: float = kwargs.pop('temperature', 0.2)
        max_tokens: int = kwargs.pop('max_tokens', 1024)
        mode: str = kwargs.pop('mode', 'chat.completion')

        logger.info(f'Using OpenAI model_id: {model_id}')

        res = self._predict(model_id=model_id,
                            sys_prompt=sys_prompt,
                            user_prompt=user_prompt,
                            temperature=temperature,
                            max_tokens=max_tokens,
                            mode=mode)

        return res

    def _predict(self,
                 model_id,
                 sys_prompt,
                 user_prompt,
                 temperature,
                 max_tokens,
                 mode: str = 'chat.completion',) -> dict:

        res = {}
        openai.api_key = self.api_key

        for i in range(self.MAX_RETRIES):
            try:
                if mode == 'chat.completion':
                    resp = openai.ChatCompletion.create(
                        model=model_id,
                        messages=[{
                            'role': 'system',
                            'content': sys_prompt
                        }, {
                            'role': 'user',
                            'content': user_prompt
                        }],
                        temperature=temperature,
                        max_tokens=max_tokens)

                    if resp:
                        ans_text = resp['choices'][0]['message']['content']
                        model_id = resp['model']
                    else:
                        logger.warning(
                            f'OpenAI GPT API call failed: got empty response '
                            f'for input {sys_prompt} {user_prompt}')
                        ans_text = ''
                        model_id = ''

                    res['ans_text'] = ans_text
                    res['model_id'] = model_id
                else:
                    raise ValueError(f'Invalid mode: {mode}')

                return res

            except Exception as e:
                logger.warning(f'OpenAI API call failed: {e}')
                time.sleep(3)
        logger.error(
            f'OpenAI API call failed after {self.MAX_RETRIES} retries')
        return res



# Copyright (c) Alibaba, Inc. and its affiliates.

from llmuses.benchmarks.race.race_adapter import DATASET_ID, SUBJECT_MAPPING, SUBSET_LIST, RACEAdapter
from llmuses.benchmarks.race.race_adapter import RACEAdapter as DataAdapterClass
from llmuses.models.model_adapter import MultiChoiceModelAdapter as ModelAdapterClass    # noqa



# Copyright (c) Alibaba, Inc. and its affiliates.
# Copyright (c) EleutherAI Inc, and its affiliates.
import csv
import os
from typing import List
import numpy as np

from llmuses.benchmarks.data_adapter import DataAdapter
from llmuses.metrics.metrics import exact_match, weighted_mean
from llmuses.utils.logger import get_logger
# flake8: noqa

logger = get_logger()


DATASET_ID = 'modelscope/trivia_qa'
SUBSET_LIST = ['default']


class TriviaQaAdapter(DataAdapter):

    def __init__(self,
                 subset_list: list = None,
                 metric_list: list = None,
                 few_shot_num: int = None,
                 train_split: str = 'dev',
                 eval_split: str = 'test',
                 **kwargs):

        if subset_list is None:
            subset_list = SUBSET_LIST

        if metric_list is None:
            metric_list = [{'name': 'WeightedAverageAccuracy', 'object': weighted_mean}]

        if few_shot_num is None:
            logger.info(f'few_shot_num is not specified for TriviaQA, use default value: 5')
            few_shot_num = 5

        super().__init__(subset_list=subset_list,
                         metric_list=metric_list,
                         few_shot_num=few_shot_num,
                         train_split=train_split,
                         eval_split=eval_split,
                         **kwargs)

    def load_from_disk(self, dataset_name_or_path, subset_list, work_dir, **kwargs) -> dict:
        data_dict = {}
        for subset_name in subset_list:
            data_dict[subset_name] = {}
            for split in [self.train_split, self.eval_split]:
                if os.path.exists(dataset_name_or_path):
                    file_path = os.path.join(dataset_name_or_path, f'trivia-{split}.qa.csv')
                else:
                    file_path = os.path.join(work_dir, dataset_name_or_path, f'trivia-{split}.qa.csv')
                if os.path.exists(file_path):
                    with open(file_path, 'r', encoding='utf-8') as f:
                        reader = csv.reader(f, delimiter='\t')
                        split_data = []
                        for row in reader:
                            assert len(row) == 2
                            question = row[0]
                            answers = eval(row[1])
                            split_data.append({
                                'input': [
                                    {"role": "system", "content": "Follow the given examples and answer the question."},
                                    {"role": "user", "content": question}
                                ],
                                'ideal': answers
                            })
                        data_dict[subset_name][split] = split_data

        return data_dict

    def gen_prompt(self, input_d: dict, subset_name: str, few_shot_list: list, **kwargs) -> dict:
        """
        Generate model prompt from raw input, unify the prompt format for TriviaQA benchmark.

        Args:
            input_d (dict): The raw input. A single data format of the TriviaQA:

            {
                "input": [
                    {"role": "system", "content": "Follow the given examples and answer the question."},
                    {"role": "user", "content": "Which Lloyd Webber musical premiered in the US on 10th December 1993?"}
                ],
                "ideal": [
                    "Sunset Blvd",
                    "West Sunset Boulevard",
                    "Sunset Boulevard",
                    "Sunset Bulevard",
                    "Sunset Blvd.",
                    "sunset boulevard",
                    "sunset bulevard",
                    "west sunset boulevard",
                    "sunset blvd"
                ]
            }

        Returns:
            {'data': [(context, continuation), ...]}
        """
        def get_sys_prompt(inp: dict) -> str:
            return inp['input'][0]['content']

        prompt = get_sys_prompt(input_d)
        few_shot_prompts = [self._generate_prompt(input_d=sample, include_answer=True) for sample in few_shot_list]
        context: str = '\n'.join(few_shot_prompts) + '\n'
        context += self._generate_prompt(input_d=input_d, include_answer=False)
        full_prompt = prompt + context

        return {'data': [full_prompt]}

    def get_gold_answer(self, input_d: dict) -> list:
        # Get the gold choice
        ans: list = input_d.get("ideal", [])
        return ans

    def parse_pred_result(self, result: str, raw_input_d: dict = None, eval_type: str = 'checkpoint') -> str:
        """
        Parse the model output to get the answer.

        Args:
            result: Predicted answer from the model. A list of loglikelihood values for inputs pairs.
            raw_input_d: The raw input. A single data format of the TriviaQA:
            eval_type: The type of evaluation, e.g. 'checkpoint' or 'service' or 'custom'.

        Returns:
            The predicted answer.
        """
        if eval_type == 'checkpoint':
            return result
        elif eval_type == 'service':  # TODO: to be implemented
            return result
        elif eval_type == 'custom':  # TODO: to be implemented
            return result
        else:
            raise ValueError(f'Unknown eval_type: {eval_type}')

    def match(self, gold: list, pred: str) -> float:
        return max([exact_match(gold=ref, pred=pred) for ref in gold])

    def compute_metric(self, review_res_list: list) -> float:
        """
        Compute evaluation result by specific metric.

        Args:
            review_res_list: review score list, e.g. [0, 1, 1, 0, ...]

        Returns:
            The metric score.
        """
        items = [(score, 1.0) for score in review_res_list]
        return weighted_mean(items)

    def gen_report(self, subset_score_map: dict, report_name: str = None) -> dict:
        """
        Generate the report for the model output.

        Args:
            subset_score_map: {subset_name: (score, num), ...}
            report_name: The user-defined report name.

        Returns:
        {
            "name":"TriviaQA",
            "metric":"WeightedAverageAccuracy",
            "score":0.3389,
            "category":[
                {
                    "name":"DEFAULT",
                    "score":0.3389,
                    "subset":[
                        {
                            "name":"default",
                            "score":0.3389
                        }
                    ]
                }
            ],
            "total_num":100
        }
        """
        total_num: int = sum([num for _, num in subset_score_map.values()])
        weighted_avg_acc: float = sum([score * num for score, num in subset_score_map.values()]) / total_num
        cate_avg_list = [{'name': subset_name, 'score': score} for subset_name, (score, _) in subset_score_map.items()]

        category_d = dict(name='DEFAULT',
                          score=weighted_avg_acc,
                          subset=cate_avg_list)

        res_map = dict(name=report_name or 'trivia_qa',
                       metric=self.metric_list[0]['name'],
                       score=weighted_avg_acc,
                       category=[category_d],
                       total_num=total_num)

        return res_map

    @classmethod
    def _generate_prompt(cls, input_d: dict, include_answer=True) -> str:

        example: str = f"Question: {input_d['input'][1]['content']}\nAnswer:"
        if include_answer:
            example += f" {input_d['ideal'][0]}\n\n"

        return example



# Copyright (c) Alibaba, Inc. and its affiliates.

import os
import copy
from dataclasses import dataclass, asdict, field
from typing import Optional, List

from llmuses.constants import DEFAULT_ROOT_CACHE_DIR
from llmuses.models.custom import CustomModel
from llmuses.utils import yaml_to_dict
from llmuses.utils.logger import get_logger

logger = get_logger()

cur_path = os.path.dirname(os.path.abspath(__file__))

registry_tasks = {
    'arc': yaml_to_dict(os.path.join(cur_path, 'registry/tasks/arc.yaml')),
    'gsm8k': yaml_to_dict(os.path.join(cur_path, 'registry/tasks/gsm8k.yaml')),
    'mmlu': yaml_to_dict(os.path.join(cur_path, 'registry/tasks/mmlu.yaml')),
    'ceval': yaml_to_dict(os.path.join(cur_path, 'registry/tasks/ceval.yaml')),
    'bbh': yaml_to_dict(os.path.join(cur_path, 'registry/tasks/bbh.yaml')),
    'general_qa': yaml_to_dict(os.path.join(cur_path, 'registry/tasks/general_qa.yaml')),

    # 'bbh_mini': yaml_to_dict(os.path.join(cur_path, 'registry/tasks/bbh_mini.yaml')),
    # 'mmlu_mini': yaml_to_dict(os.path.join(cur_path, 'registry/tasks/mmlu_mini.yaml')),
    # 'ceval_mini': yaml_to_dict(os.path.join(cur_path, 'registry/tasks/ceval_mini.yaml')),

}


@dataclass
class TaskConfig:
    model_args: Optional[dict] = field(default_factory=dict)
    generation_config: Optional[dict] = field(default_factory=dict)
    dataset_args: Optional[dict] = field(default_factory=dict)
    dry_run: bool = False
    model: CustomModel = None
    eval_type: str = 'custom'
    datasets: list = field(default_factory=list)
    work_dir: str = DEFAULT_ROOT_CACHE_DIR
    outputs: str = None
    mem_cache: bool = False
    use_cache: bool = True
    stage: str = 'all'      # `all` or `infer` or `review`
    dataset_hub: str = 'ModelScope'
    dataset_dir: str = ''
    limit: int = None

    # def __post_init__(self):
    #     self.registry_tasks = {
    #         'arc': yaml_to_dict(os.path.join(cur_path, 'registry/tasks/arc.yaml')),
    #         'gsm8k': yaml_to_dict(os.path.join(cur_path, 'registry/tasks/gsm8k.yaml')),
    #         'mmlu': yaml_to_dict(os.path.join(cur_path, 'registry/tasks/mmlu.yaml')),
    #         'ceval': yaml_to_dict(os.path.join(cur_path, 'registry/tasks/ceval.yaml')),
    #         'bbh': yaml_to_dict(os.path.join(cur_path, 'registry/tasks/bbh.yaml')),
    #
    #         'bbh_mini': yaml_to_dict(os.path.join(cur_path, 'registry/tasks/bbh_mini.yaml')),
    #         'mmlu_mini': yaml_to_dict(os.path.join(cur_path, 'registry/tasks/mmlu_mini.yaml')),
    #         'ceval_mini': yaml_to_dict(os.path.join(cur_path, 'registry/tasks/ceval_mini.yaml')),
    #
    #     }

    @staticmethod
    def registry(name: str, data_pattern: str, dataset_dir: str = None, subset_list: list = None) -> None:
        """
        Register a new task (dataset) for evaluation.

        Args:
            name: str, the dataset name.
            data_pattern: str, the data pattern for the task.
                    e.g. `mmlu`, `ceval`, `gsm8k`, ...
                    refer to task_config.list() for all available datasets.
            dataset_dir: str, the directory to store multiple datasets files. e.g. /path/to/data, 
                then your specific custom dataset directory will be /path/to/data/{name}
            subset_list: list, the subset list for the dataset.
                e.g. ['middle_school_politics', 'operating_system']
                refer to the mmlu for example.  https://github.com/hendrycks/test/blob/master/categories.py
        """
        available_datasets = list(registry_tasks.keys())
        if data_pattern not in available_datasets:
            logger.error(f'No dataset found in available datasets: {available_datasets}, got data_pattern: {data_pattern}')
            return

        # Reuse the existing task config and update the datasets
        pattern_config = registry_tasks.get(data_pattern)

        custom_config = copy.deepcopy(pattern_config)
        custom_config.update({'datasets': [data_pattern]})
        custom_config.update({'dataset_hub': 'Local'})     # TODO: to support `ModelScope`
        if 'dataset_args' in custom_config:
            if data_pattern not in custom_config:
                custom_config['dataset_args'].update({data_pattern: {}})
        else:
            custom_config.update({'dataset_args': {data_pattern: {}}})

        if dataset_dir is not None:
            custom_config['dataset_args'][data_pattern].update({'local_path': dataset_dir})

        if subset_list is not None:
            # custom_config['dataset_args'].get(data_pattern, {}).update({'subset_list': subset_list})
            custom_config['dataset_args'][data_pattern].update({'subset_list': subset_list})

        registry_tasks.update({name: custom_config})
        logger.info(f'** Registered task: {name} with data pattern: {data_pattern}')

    def to_dict(self):
        # Note: to avoid serialization error for some model instance
        _tmp_model = copy.copy(self.model)
        self.model = None
        res_dict = asdict(self)
        res_dict.update({'model': _tmp_model})
        self.model = _tmp_model

        return res_dict

    @staticmethod
    def load(custom_model: CustomModel, tasks: List[str]) -> List['TaskConfig']:
        res_list = []
        for task_name in tasks:
            task: dict = registry_tasks.get(task_name, None)
            if task is None:
                logger.error(f'No task found in tasks: {list(registry_tasks.keys())}, got task_name: {task_name}')
                continue

            res = TaskConfig(**task)
            res.model = custom_model
            if res.outputs is None:
                res.outputs = os.path.join(res.work_dir,
                                           'outputs',
                                           f"eval_{'-'.join(tasks)}_{res.model.config['model_id']}_{res.model_args.get('revision', 'default')}")
            res_list.append(res)

        return res_list

    @staticmethod
    def list():
        return list(registry_tasks.keys())


class TempModel(CustomModel):

    def __init__(self, config: dict):
        super().__init__(config=config)

    def predict(self, prompt: str, **kwargs):
        return prompt + ': response'


if __name__ == '__main__':
    model = TempModel(config={'model_id': 'test-swift-dummy-model'})
    task_config = TaskConfig()

    # Register a new task
    TaskConfig.registry(name='arc_swift', data_pattern='arc', dataset_dir='/Users/jason/workspace/work/maas/benchmarks/swift_custom_work')

    import json
    swift_eval_task: List[TaskConfig] = TaskConfig.load(custom_model=model, tasks=['gsm8k', 'arc', 'arc_swift'])
    for item in swift_eval_task:
        print(item.to_dict())
        print()




# Copyright (c) Alibaba, Inc. and its affiliates.
from abc import ABC, abstractmethod
from typing import Any, Union, Dict
import torch


class CustomModel(ABC):

    def __init__(self, config: dict, **kwargs):
        self.config = config
        self.kwargs = kwargs

        if config.get('model_id', None) is None:
            raise ValueError(f"**Error: model_id is required in config for CustomModel. Got config: {config}")

    @abstractmethod
    @torch.no_grad()
    def predict(self, prompt: str, **kwargs) -> Dict[str, Any]:
        """
        Model prediction function.

        Args:
            prompt (str): The input prompt for the model.

            **kwargs: kwargs

        Returns:
            res (dict): The model prediction results. Format:
            {
              'choices': [
                {
                  'index': 0,
                  'message': {
                    'content': 'xxx',
                    'role': 'assistant'
                  }
                }
              ],
              'created': 1677664795,
              'model': 'gpt-3.5-turbo-0613',   # should be model_id
              'object': 'chat.completion',
              'usage': {
                'completion_tokens': 17,
                'prompt_tokens': 57,
                'total_tokens': 74
              }
            }
        """
        raise NotImplementedError



# Copyright (c) Alibaba, Inc. and its affiliates.

from llmuses.benchmarks.gsm8k.gsm8k_adapter import DATASET_ID, SUBSET_LIST
from llmuses.benchmarks.gsm8k.gsm8k_adapter import GSM8KAdapter as DataAdapterClass
from llmuses.models.model_adapter import ChatGenerationModelAdapter as ModelAdapterClass    # noqa



# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import json

import datasets
import pandas as pd


_CITATION = """\
@article{2017arXivtriviaqa,
       author = {{Joshi}, Mandar and {Choi}, Eunsol and {Weld},
                 Daniel and {Zettlemoyer}, Luke},
        title = "{triviaqa: A Large Scale Distantly Supervised Challenge Dataset for Reading Comprehension}",
      journal = {arXiv e-prints},
         year = 2017,
          eid = {arXiv:1705.03551},
        pages = {arXiv:1705.03551},
archivePrefix = {arXiv},
       eprint = {1705.03551},
}
"""

_DESCRIPTION = """\
TriviaqQA is a reading comprehension dataset containing over 650K question-answer-evidence triples.
"""

_HOMEPAGE = "https://modelscope.cn/datasets/modelscope/trivia_qa/summary"

_URL = "https://modelscope.cn/api/v1/datasets/modelscope/trivia_qa/repo?Revision=master&FilePath=trivia_qa.zip"

task_list = [
    "default"
]


class TriviaQAConfig(datasets.BuilderConfig):
    def __init__(self, **kwargs):
        super().__init__(version=datasets.Version("1.0.0"), **kwargs)


class TriviaQA(datasets.GeneratorBasedBuilder):
    BUILDER_CONFIGS = [
        TriviaQAConfig(
            name=task_name,
        )
        for task_name in task_list
    ]

    def _info(self):
        features = datasets.Features(
            {
                "input": [{
                    "role": datasets.features.Value("string"),
                    "content": datasets.features.Value("string"),
                }],
                "ideal": [datasets.Value("string")],
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        data_dir = dl_manager.download_and_extract(_URL)
        task_name = self.config.name
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": os.path.join(
                        data_dir, f"trivia_qa/test.jsonl"
                    ),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split("dev"),
                gen_kwargs={
                    "filepath": os.path.join(
                        data_dir, f"trivia_qa/dev.jsonl"
                    ),
                },
            ),
        ]

    def _generate_examples(self, filepath):
        with open(filepath, encoding='utf-8') as f:
            contents = [json.loads(line) for line in f.readlines()]
            for i, instance in enumerate(contents):
                yield i, instance


# Copyright (c) Alibaba, Inc. and its affiliates.
import glob
import os.path

from llmuses.benchmarks.data_adapter import DataAdapter
from llmuses.metrics.metrics import bleu_ngram_one_sample, weighted_mean
from llmuses.metrics.rouge_metric import compute_rouge_score_one_sample_zh
from llmuses.utils import jsonl_to_list
from llmuses.utils.logger import get_logger
from typing import Any, Optional
from collections import defaultdict
import json

logger = get_logger()

DATASET_ID = 'general_qa'
SUBSET_LIST = ['default']


class GeneralQAAdapter(DataAdapter):
    # TODO: set few_shot_num

    def __init__(self,
                 subset_list: list = None,
                 metric_list: list = None,
                 train_split: str = None,
                 eval_split: str = 'test',
                 **kwargs):
        if subset_list is None:
            subset_list = SUBSET_LIST

        if metric_list is None:
            metric_list = [{'name': 'WeightedAverageBLEU', 'object': weighted_mean}]
        
        super().__init__(subset_list=subset_list,
                         metric_list=metric_list,
                         train_split=train_split,
                         eval_split=eval_split,
                         **kwargs)
    
    def load(self,
             dataset_name_or_path: str,
             subset_list: list = None,
             **kwargs) -> dict:

        data_file_list = glob.glob(os.path.join(dataset_name_or_path, '*.jsonl'))
        data_list = []

        try:
            for file_path in data_file_list:
                data_list.extend(jsonl_to_list(file_path))
        except Exception as e:
            raise ValueError(f"Failed to load data from {dataset_name_or_path}, got error: {e}")

        data_dict = {'default': {'test': data_list}}

        return data_dict
    
    def gen_prompt(self, input_d: dict, subset_name: str, few_shot_list: list, **kwargs) -> dict:
        """
        Args:
            input_d:
                format1: {'history': [['q1', 'a1'], ['q2', 'a2']], 'question': '', 'answer': ''}
                format2: {'history': [['q1', 'a1'], ['q2', 'a2']], 'query': '', 'response': ''}

        Returns:
            {'data': [prompt]}

        """
        # prompt = f"'<|im_start|>user\n{input_d['input']}<|im_end|>\n<|im_start|>assistant\n'"
        history = input_d.get('history', [])    # history: [['q1', 'a1'], ['q2', 'a2'], ...]
        if len(history) > 0:
            logger.warning(f"The history is not included in the prompt for GeneralQA. To be supported in the future.")

        prompt = input_d.get('question', '') or input_d.get('query', '')

        # if len(history) > 0:
        #     prompt = '\n'.join(history) + '\n' + prompt
        return {'data': [prompt]}
    
    def get_gold_answer(self, input_d: dict) -> str:
        """
        Args:
            input_d: {'history': [], 'question': '', 'answer': ''}

        Returns:
            gold_answer: str

        """
        return input_d.get('answer', '') or input_d.get('response', '')
    
    def parse_pred_result(self, result: str, raw_input_d: dict = None, eval_type: str = 'checkpoint') -> str:
        """
        Args:
            result: str

        Returns:
            pred_result: str

        """
        return result
    
    def match(self, gold: str, pred: str) -> float:
        """
        Args:
            gold: str
            pred: str

        Returns:
            bleu_score: float

        """
        item = [(gold, pred)]
        res = dict()
        rouge_dict = compute_rouge_score_one_sample_zh([pred], [gold])
        bleu_dict = bleu_ngram_one_sample(pred, gold)
        res.update(rouge_dict)
        res.update(bleu_dict)
        # return bleu(item)
        return res
    
    def compute_metric(self, review_res_list: list) -> float:
        """
        compute weighted mean of the bleu score of all samples

        Args:
            review_res_list: [score1, score2, ...]

        Returns:
            avg_res: float

        """
        items = defaultdict(list)
        for scores in review_res_list:
            for k,v in scores.items():
                items[k].append((v, 1.0))
        # items = [(score, 1.0) for score in review_res_list]
        res = {k: weighted_mean(v) for k,v in items.items()}
        # return weighted_mean(items)
        return res
    
    def gen_report(self, subset_score_map: dict, report_name: str = None) -> dict:
        """
        Args:
            subset_score_map: {subset_name: (score_dict, num), ...}
            report_name: str, the user-defined report name.

        Returns:
        {
            "name":"GeneralQA",
            "metric":"WeightedAverageBLEU",
            "score":0.399,
            "category":[
                {
                    "name":"DEFAULT",
                    "score":0.399,
                    "subset":[
                        {
                            "name":"default",
                            "score":0.399
                        },
                    ]
                }
            ],
            "total_num":10
        }
        """
        total_num: int = sum([num for _, num in subset_score_map.values()])
        # weighted_avg_bleu: float = sum([score * num for score, num in subset_score_map.values()]) / total_num
        cate_avg_list = [{'name': subset_name, 'score': score_dict} for subset_name, (score_dict, _) in subset_score_map.items()]
        total_avg_list = defaultdict(float)
        for score_dict, num in subset_score_map.values():
            for metric, score in score_dict.items():
                total_avg_list[metric] += score * num / total_num

        category_d = dict(name="DEFAULT",
                          score=total_avg_list,
                          subset=cate_avg_list)
        
        res_map = dict(name=report_name or "general_qa",
                       metric=self.metric_list[0]['name'],
                       score=total_avg_list,
                       category=[category_d],
                       total_num=total_num)
        
        return res_map


# Copyright (c) Alibaba, Inc. and its affiliates.

from llmuses.benchmarks.truthful_qa.truthful_qa_adapter import TruthfulQaAdapter, DATASET_ID, SUBSET_LIST
from llmuses.benchmarks.truthful_qa.truthful_qa_adapter import TruthfulQaAdapter as DataAdapterClass
from llmuses.models.model_adapter import ContinuationLogitsModelAdapter as ModelAdapterClass    # noqa



# Copyright (c) Alibaba, Inc. and its affiliates.

import os
import json
from llmuses.benchmarks.data_adapter import DataAdapter
from llmuses.metrics.metrics import exact_match, weighted_mean
from llmuses.utils import normalize_score, jsonl_to_list
from llmuses.utils.logger import get_logger
# flake8: noqa

logger = get_logger()

DATASET_ID = 'modelscope/race'

SUBSET_LIST = [
    "high",
    "middle"
]


SUBJECT_MAPPING = {"high": "High",
                   "middle": "Middle"
}


class RACEAdapter(DataAdapter):

    choices = ['A', 'B', 'C', 'D']

    def __init__(self,
                 subset_list: list = None,
                 metric_list: list = None,
                 few_shot_num: int = None,
                 train_split: str = 'train',
                 eval_split: str = 'test',
                 **kwargs):

        if subset_list is None:
            subset_list = SUBSET_LIST

        if metric_list is None:
            metric_list = [{'name': 'WeightedAverageAccuracy', 'object': weighted_mean}]

        if few_shot_num is None:
            logger.info(f'Set 3-shot examples by system for RACE.')
            few_shot_num = 3

        if few_shot_num > 3:
            logger.warning(f'few_shot_num <= 3 for RACE, but got {few_shot_num}. Use 3-shot by default.')
            few_shot_num = 3

        super().__init__(subset_list=subset_list,
                         metric_list=metric_list,
                         few_shot_num=few_shot_num,
                         train_split=train_split,
                         eval_split=eval_split,
                         **kwargs)

    def load_from_disk(self, dataset_name_or_path, subset_list, work_dir, **kwargs) -> dict:
        data_dict = {}
        for subset_name in subset_list:
            data_dict[subset_name] = {}
            for split in [self.train_split, self.eval_split]:
                if os.path.exists(dataset_name_or_path):
                    file_path = os.path.join(dataset_name_or_path, subset_name, f'{split}.jsonl')
                else:
                    file_path = os.path.join(work_dir, dataset_name_or_path, subset_name, f'{split}.jsonl')
                if os.path.exists(file_path):
                    data_dict[subset_name][split] = jsonl_to_list(file_path)

        return data_dict

    def gen_prompt(self, input_d: dict, subset_name: str, few_shot_list: list, **kwargs) -> dict:
        """
        Generate model prompt from raw input, unify the prompt format for RACE benchmark.

        Args:
            input_d (dict): The raw input. A single data format of the RACE:

            {'example_id': 'high3680.txt',
            'article': 'Astronauts on shorter shuttle missions often work very long days. Tasks are scheduled so tightly that break times are often used to finish the day's work. This type of schedule is far too demanding for long missions on the International Space Station(ISS). ISS crewmembers usually live in space for at least a quarter of a year. They work five days on and two days off to _ the normal way they do things on Earth as much as possible. Weekends give the crew valuable time to rest and do a few hours of housework. They can communicate with family and friends by email , internet phone and through private video conferences. While astronauts cannot go to a baseball game or a movie in orbit, there are many familiar activities that they can still enjoy . Before a mission, the family and friends of each ISS crewmember put together a collection of family photos, messages, videos and reading material for the astronauts to look at when they will be floating 370 kilometers above the Earth. During their mission, the crew also receives care packages with CDs, books, magazines, photos and letters . And as from early 2010, the internet became available on the ISS , giving astronauts the chance to do some "web surfing "in their personal time. Besides relaxing with these more common entertainments, astronauts can simply enjoy the experience of living in space. Many astronauts say that one of the most relaxing things to do in space is to look out the window and stare at the universe and the Earth's vast land mass and oceans.',
            'answer': 'C',
            'question': 'The passage mainly discusses how astronauts _ .',
            'options': [
                "work for longer missions in space",
                "connect with people on the Earth",
                "spend their free time in space",
                "observe the Earth from space"]}

        Returns:
            {'data': [(context, continuation), ...]}

        """
        prompt = 'The following are multiple choice reading comprehension questions (with answers).\n\n'.format(
            self._format_subject(subset_name)
        )
        few_shot_prompts = [self._generate_prompt(input_d=sample, include_answer=True) for sample in few_shot_list]

        context: str = '\n'.join(few_shot_prompts) + '\n'
        context += self._generate_prompt(input_d=input_d, include_answer=False)
        context = prompt + context

        full_prompt: str = context.strip() + self._generate_prompt(input_d=input_d, include_answer=False)

        return {'data': [full_prompt], 'multi_choices': self.choices}

    def get_gold_answer(self, input_d: dict) -> str:
        # Get the gold choice
        return input_d.get('answer', '')

    def parse_pred_result(self, result: str, raw_input_d: dict = None, eval_type: str = 'checkpoint') -> str:
        """
        Parse the model output to get the answer. Could be the best choice index.

        Args:
            result: Predicted answer from the model. Usually a string for chat.
            raw_input_d: The raw input. Depending on the dataset.
            eval_type: The evaluation type. e.g. 'checkpoint' or 'service' or 'custom'.

        Returns:
            The parsed answer. Depending on the dataset. Usually a string for chat.
        """
        if eval_type == 'checkpoint':
            return result
        elif eval_type == 'service':        # TODO: to be implemented
            return result
        elif eval_type == 'custom':         # TODO: to be implemented
            return result
        else:
            raise ValueError(f'Unknown eval_type: {eval_type}')

    def match(self, gold: str, pred: str) -> float:
        return exact_match(gold=gold, pred=pred)

    def compute_metric(self, review_res_list: list) -> float:
        """
        Compute evaluation result by specific metric.

        Args:
            review_res_list: review score list, e.g. [0, 1, 1, 0, ...]

        Returns:
            The metric score.
        """
        items = [(score, 1.0) for score in review_res_list]
        return weighted_mean(items)

    def gen_report(self, subset_score_map: dict, report_name: str = None) -> dict:
        """
        Generate report for the evaluation.

        Args:
            subset_score_map: The subset-score mapping. e.g. {subset_name: (score, num), ...}
            report_name: The user-defined report name.

        Returns:
        {
            "name":"RACE",
            "metric":"WeightedAverageAccuracy",
            "score":0.3389,
            "category":[
                {
                   "name":"High",
                   "score":0.2528,
                   "subset":[
                       {
                           "name":"high",
                           "score":0.2528
                       }
                   ]
                }
            ],
            "total_num":59
        }
        """
        total_num: int = sum([num for _, num in subset_score_map.values()])
        weighted_avg_acc: float = sum([score * num for score, num in subset_score_map.values()]) / total_num

        # Get domain-subject mapping
        subject_review_map = {}
        for subset_name, (subset_score, num) in subset_score_map.items():
            domain_name: str = SUBJECT_MAPPING.get(subset_name)
            if domain_name in subject_review_map:
                subject_review_map[domain_name].append((subset_name, subset_score, num))
            else:
                subject_review_map[domain_name] = [(subset_name, subset_score, num)]

        # Get domain score
        category_list = []
        for domain_name, domain_res_list in subject_review_map.items():
            domain_weighted_avg_acc = sum([score * num for _, score, num in domain_res_list]) / \
                                     sum([num for _, _, num in domain_res_list])
            domain_weighted_avg_acc = normalize_score(score=domain_weighted_avg_acc)
            category_list.append({'name': domain_name,
                                  'score': normalize_score(score=domain_weighted_avg_acc),
                                  'subset': [{'name': subset_name, 'score': subset_score}
                                             for subset_name, subset_score, _ in domain_res_list]})

        # Get final dict of report
        res_map = dict(name=report_name or 'race',
                       metric=self.metric_list[0]['name'],
                       score=weighted_avg_acc,
                       category=category_list,
                       total_num=total_num)

        return res_map

    @classmethod
    def _generate_prompt(cls, input_d: dict, include_answer=True) -> str:

        input_choices: list = input_d['options']

        example: str = 'Article:\n{}\nQuestion:\n{}'.format(input_d['article'], input_d['question'])
        for j in range(len(cls.choices)):
            example += '\n{}. {}'.format(cls.choices[j], input_choices[j])

        example += '\nAnswer:'
        if include_answer:
            example += ' {}\n\n'.format(input_d['answer'])

        return example

    @classmethod
    def _format_subject(cls, subject):
        l = subject.split('_')
        s = ''
        for entry in l:
            s += ' ' + entry
        return s



# Copyright (c) Alibaba, Inc. and its affiliates.

import os
import re
import random
import json

from llmuses.benchmarks.data_adapter import DataAdapter
from llmuses.constants import AnswerKeys
from llmuses.metrics.metrics import exact_match, weighted_mean
from llmuses.utils import normalize_score, ResponseParser
from llmuses.utils.logger import get_logger
# flake8: noqa

logger = get_logger()

DATASET_ID = 'modelscope/bbh'


# BBH multiple choice subset list
MULTIPLE_CHOICE = 'multiple_choice'
MULTIPLE_CHOICE_LIST = [
    'temporal_sequences',
    'disambiguation_qa',
    'date_understanding',
    'tracking_shuffled_objects_three_objects',
    'penguins_in_a_table',
    'geometric_shapes',
    'snarks',
    'ruin_names',
    'tracking_shuffled_objects_seven_objects',
    'tracking_shuffled_objects_five_objects',
    'logical_deduction_three_objects',
    'hyperbaton',
    'logical_deduction_five_objects',
    'logical_deduction_seven_objects',
    'movie_recommendation',
    'salient_translation_error_detection',
    'reasoning_about_colored_objects',
]

# The free form subset list of BBH dataset
FREE_FORM = 'free_form'
FREE_FORM_LIST = [
    'multistep_arithmetic_two',
    'navigate',
    'dyck_languages',
    'word_sorting',
    'sports_understanding',
    'boolean_expressions',
    'object_counting',
    'formal_fallacies',
    'causal_judgement',
    'web_of_lies',
]

# BBH sub-task type
TASK_TYPE = 'task_type'
SUBSET_LIST = MULTIPLE_CHOICE_LIST + FREE_FORM_LIST


class BBHAdapter(DataAdapter):
    """
    Adapter for BBH free-form and multiple-choices sub-tasks.
    """

    def __init__(self,
                 subset_list: list = None,
                 metric_list: list = None,
                 few_shot_num: int = None,
                 train_split: str = None,
                 eval_split: str = 'test',
                 **kwargs):

        if subset_list is None:
            subset_list = SUBSET_LIST

        if metric_list is None:
            metric_list = [{'name': 'WeightedAverageAccuracy', 'object': weighted_mean}]

        if few_shot_num is None:
            logger.info(f'Set 3-shot examples by system for BBH.')
            few_shot_num = 3

        if few_shot_num != 3 and few_shot_num != 0:
            logger.error(f'BBH uses 3-shot examples with CoT or 0-shot by system, but got {few_shot_num}. '
                         f'Use 3-shot by default.')
            few_shot_num = 3

        super().__init__(subset_list=subset_list,
                         metric_list=metric_list,
                         few_shot_num=few_shot_num,
                         train_split=train_split,
                         eval_split=eval_split,
                         **kwargs)

    def load_from_disk(self, dataset_name_or_path, subset_list, work_dir, **kwargs) -> dict:
        data_dict = {}
        for subset_name in subset_list:
            for split_name in [self.eval_split]:
                if os.path.exists(dataset_name_or_path):
                    file_path = os.path.join(dataset_name_or_path, f'{subset_name}.json')
                else:
                    file_path: str = os.path.join(work_dir, dataset_name_or_path, f'{subset_name}.json')
                if os.path.exists(file_path):
                    with open(file_path, 'r') as f:
                        examples = json.load(f)['examples']
                        if subset_name in data_dict:
                            data_dict[subset_name].update({split_name: examples})
                        else:
                            data_dict[subset_name] = {split_name: examples}

        return data_dict

    def gen_prompt(self, input_d: dict, few_shot_list: list, **kwargs) -> dict:
        """
        Generate model prompt from raw data, unify the prompt format for bbh(multiple choice) benchmark.

        Args:
            input_d (dict): The raw input. A single data format of the BBH:

            {
                'input': '((-1 + 2 + 9 * 5) - (-2 + -4 + -4 * -7)) =',
                'target': '24',
            }

        Returns:
            {'data': ['xxx']}
        """
        # few_shot_list: should be ['xxxx']
        cot_prompts: str = few_shot_list[0] if len(few_shot_list) > 0 else ''
        full_prompt: str = f"Follow the given examples and answer the question.\n{cot_prompts}\n\nQ: {input_d['input']}\nA: Let's think step by step."

        return {'data': [full_prompt]}

    def gen_prompts(self, data_dict: dict) -> dict:
        """
        Generate dataset prompts from raw input, unify the prompt format for different datasets.

        Args:
            data_dict:  Refer to the output of load method: llmuses.benchmarks.benchmark.Benchmark.load

        Returns:
            {'subset_name': [prompt_d_1, prompt_d_2, ...]}
            prompt_d_i (dict): refer to the output of gen_prompt method.

        e.g. train -- few-shot data, test -- target dataset to evaluate.
        """
        res_dict: dict = {}

        if self.few_shot_num < 0:
            raise ValueError(f'Invalid shot_num: {self.few_shot_num} for few-shot evaluation.')

        logger.info(f'\n** Use default settings: \n'
                    f'>few_shot_num: {self.few_shot_num}, '
                    f'>few_shot_split: {self.train_split}, '
                    f'>target_eval_split: {self.eval_split}')

        for sub_name, sub_data_dict in data_dict.items():
            few_shot_data = []
            if self.few_shot_num > 0:
                with open(os.path.join(os.path.dirname(__file__), 'cot_prompts', f'{sub_name}.txt'), 'r') as f:
                    cot_prompt_str = f.read()
                few_shot_data = [cot_prompt_str]

            res_dict[sub_name] = []
            for sample_d in sub_data_dict[self.eval_split]:
                prompt_d = self.gen_prompt(input_d=sample_d, few_shot_list=few_shot_data)
                sample_d_new = sample_d.copy()
                if sub_name in MULTIPLE_CHOICE_LIST:
                    sample_d_new[TASK_TYPE] = MULTIPLE_CHOICE
                elif sub_name in FREE_FORM_LIST:
                    sample_d_new[TASK_TYPE] = FREE_FORM
                else:
                    raise ValueError(f'Invalid subset name: {sub_name}')

                prompt_d[AnswerKeys.RAW_INPUT] = sample_d_new
                res_dict[sub_name].append(prompt_d)

        rnd = random.Random()
        rnd.seed(42)
        for k, v in res_dict.items():
            rnd.shuffle(v)

        return res_dict

    def get_gold_answer(self, input_d: dict) -> str:
        # Get the gold choice
        gold = input_d.get('target')
        if gold is None:
            logger.error(f'BBHAdapter: gold is None.')
        return gold

    def parse_pred_result(self, result: str, raw_input_d: dict = None, eval_type: str = 'checkpoint') -> str:
        """
        Parse the model output to get the answer. Could be the best choice index.

        Args:
            result: Predicted answer from the model. Usually a string for chat.
            raw_input_d (dict): The raw input. Depending on the dataset.
            eval_type: 'checkpoint' or 'service' or `custom`, default: 'checkpoint'

        Returns:
            The parsed answer. Depending on the dataset. Usually a string for chat.
        """
        # Note: to use same extraction method for both of checkpoint/service/custom.
        task_type: str = raw_input_d.get(TASK_TYPE)

        if task_type == MULTIPLE_CHOICE:
            return self._extract_mc_answer(result)
        elif task_type == FREE_FORM:
            return self._extract_ff_answer(result)
        else:
            raise ValueError(f'Invalid task type: {task_type}')

    def match(self, gold: str, pred: str) -> float:
        return exact_match(gold=gold, pred=pred)

    def compute_metric(self, review_res_list: list) -> float:
        """
        Compute evaluation result by specific metric.

        Args:
            review_res_list: review score list, e.g. [0, 1, 1, 0, ...]

        Returns:
            The metric score.
        """
        items = [(score, 1.0) for score in review_res_list]
        return weighted_mean(items)

    def gen_report(self, subset_score_map: dict, report_name: str = None) -> dict:
        """
        Generate the report for the model output.

        Args:
            subset_score_map: The subset-score mapping. e.g. {subset_name: (score, num), ...}
            report_name: The user-defined report name.

        Returns: A dict of metric calculation results. The format is like:
        {
            "name":"BBH",
            "metric":"WeightedAverageAccuracy",
            "score":0.3389,
            "category":[
                {
                    "name":"DEFAULT",
                    "score":0.3389,
                    "subset":[
                        {
                            "name":"BBH",
                            "score":0.3389
                        },
                    ]
                }
            ],
            "total_num":100
        }
        """
        total_num: int = sum([num for _, num in subset_score_map.values()])
        weighted_avg_acc: float = sum([score * num for score, num in subset_score_map.values()]) / total_num
        weighted_avg_acc = normalize_score(score=weighted_avg_acc)
        cate_avg_list = [{'name': subset_name, 'score': normalize_score(score=score)} for subset_name, (score, _) in subset_score_map.items()]

        category_d = dict(name='DEFAULT',
                          score=weighted_avg_acc,
                          subset=cate_avg_list)

        res_map = dict(name=report_name or 'bbh',
                       metric=self.metric_list[0]['name'],
                       score=weighted_avg_acc,
                       category=[category_d],
                       total_num=total_num)

        return res_map

    @classmethod
    def _extract_mc_answer(cls, ans: str) -> str:
        """
        Extract the answer from the model output for Multiple choice task.
        """
        ans_line = ans.split('answer is ')
        if len(ans_line) != 1:
            ans = ans_line[1].strip()
        match = re.search(r'\(([A-Z])\)*', ans)
        if match:
            return match.group(1)
        match = re.search(r'([A-Z])', ans)
        if match:
            return match.group(1)
        return ans

    @classmethod
    def _extract_ff_answer(cls, ans: str):
        """
        Extract the answer from the model output for Free-form task.
        """
        res = ResponseParser.parse_first_option(ans)
        if res:
            return res

        ans_line = ans.split('answer is ')
        if len(ans_line) != 1:
            ans = ans_line[1].strip()
        ans = ans.split('\n')[0]
        if ans.endswith('.'):
            ans = ans[:-1]
        return ans



# Copyright (c) Alibaba, Inc. and its affiliates.
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.

"""HellaSwag: Can a Machine Really Finish Your Sentence? is a new dataset for commonsense NLI.
    A paper was published at ACL2019.
"""

"""DO NOT EDIT."""

import json
import datasets
# flake8: noqa

# HomePage: https://rowanzellers.com/hellaswag/
# GitHub: https://github.com/rowanz/hellaswag


_CITATION = """\
@inproceedings{zellers2019hellaswag,
    title={HellaSwag: Can a Machine Really Finish Your Sentence?},
    author={Zellers, Rowan and Holtzman, Ari and Bisk, Yonatan and Farhadi, Ali and Choi, Yejin},
    booktitle ={Proceedings of the 57th Annual Meeting of the Association for Computational Linguistics},
    year={2019}
}
"""

_DESCRIPTION = """
HellaSwag: Can a Machine Really Finish Your Sentence? is a new dataset for commonsense NLI. A paper was published at ACL2019.
"""
_URL = 'https://modelscope.oss-cn-beijing.aliyuncs.com/open_data/hellaswag/'
_URLS = {
    'train': _URL + 'hellaswag_train.jsonl',
    'test': _URL + 'hellaswag_test.jsonl',
    'dev': _URL + 'hellaswag_val.jsonl',
}


class Hellaswag(datasets.GeneratorBasedBuilder):
    """TODO(hellaswag): Short description of my dataset."""

    # TODO(hellaswag): Set up version.
    VERSION = datasets.Version('0.1.0')

    def _info(self):
        # TODO(hellaswag): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    # These are the features of your dataset like images, labels ...
                    'ind': datasets.Value('int32'),
                    'activity_label': datasets.Value('string'),
                    'ctx_a': datasets.Value('string'),
                    'ctx_b': datasets.Value('string'),
                    'ctx': datasets.Value('string'),
                    'endings': datasets.features.Sequence(datasets.Value('string')),
                    'source_id': datasets.Value('string'),
                    'split': datasets.Value('string'),
                    'split_type': datasets.Value('string'),
                    'label': datasets.Value('string'),
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage='https://rowanzellers.com/hellaswag/',
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(hellaswag): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        urls_to_download = _URLS
        dl_dir = dl_manager.download_and_extract(urls_to_download)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={'filepath': dl_dir['train']},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={'filepath': dl_dir['test']},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={'filepath': dl_dir['dev']},
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        # TODO(hellaswag): Yields (key, example) tuples from the dataset
        with open(filepath, encoding='utf-8') as f:
            for id_, row in enumerate(f):
                data = json.loads(row)
                yield id_, {
                    'ind': int(data['ind']),
                    'activity_label': data['activity_label'],
                    'ctx_a': data.get('ctx_a', ''),
                    'ctx_b': data.get('ctx_b', ''),
                    'ctx': data['ctx'],
                    'endings': data.get('endings', []),
                    'source_id': data['source_id'],
                    'split': data['split'],
                    'split_type': data['split_type'],
                    'label': str(data.get('label', '')),
                }



# Copyright (c) Alibaba, Inc. and its affiliates.

import os
import json
import glob
from tabulate import tabulate

"""
Combine and generate table for reports of LLMs.
"""


def get_report(report_file: str):
    data_d: dict = json.load(open(report_file, 'r'))
    dataset_name = data_d['name']
    score = data_d['score']     # float or dict
    score_d = {}
    if isinstance(score, dict):
        # score_d = dict([(k, round(v, 4) * 100) for k, v in score.items()])
        score_d = score
    elif isinstance(score, float):
        # score_d['acc'] = round(score, 4) * 100
        score_d['acc'] = score
    else:
        raise ValueError(f'Unknown score type: {type(score)}')
    score_str = '\n'.join([str(v) + ' (' + k + ')' for k, v in score_d.items()])

    return {'dataset_name': dataset_name, 'score': score_str}


def get_model_reports(model_report_dir: str):
    model_report_dir = os.path.normpath(model_report_dir)
    model_report_dir = model_report_dir.rstrip('reports')
    model_info = os.path.basename(os.path.normpath(model_report_dir))
    model_name = '_'.join(model_info.split('_')[:-1][3:])
    report_files = glob.glob(os.path.join(model_report_dir, 'reports', '*.json'))

    model_reports_d = {model_name: []}
    for file_path in report_files:
        report_d = get_report(file_path)
        model_reports_d[model_name].append(report_d)

    return model_reports_d


def gen_table(reports_path_list: list):
    table_values = []
    headers = ['Model']
    is_headers_set = False

    for report_path in reports_path_list:
        model_reports_d = get_model_reports(report_path)
        for model_name, report_list in model_reports_d.items():
            # report_list: [{'dataset_name': 'CompetitionMath', 'score': '4.42 (acc)'},
            #               {'dataset_name': 'GSM8K', 'score': '28.51 (acc)'}]
            report_list = sorted(report_list, key=lambda x: x['dataset_name'])
            if not is_headers_set:
                headers.extend([x['dataset_name'] for x in report_list])
                is_headers_set = True
            single_row = []
            single_row.append(model_name)
            for single_report in report_list:
                # e.g. '28.51 (acc)'
                single_row.append(single_report['score'])
            table_values.append(single_row)

    report_table = tabulate(table_values, headers=headers, tablefmt='grid')
    return report_table


if __name__ == '__main__':
    report_dir_1 = '/to/path/20231129_020533_default_ZhipuAI_chatglm2-6b-base_none/reports'
    report_dir_2 = '/to/path/20231129_020533_default_ZhipuAI_chatglm2-6b_none/reports'

    report_table = gen_table([report_dir_1, report_dir_2])
    print(report_table)

    # ALL VALUES ONLY FOR EXAMPLE
    # +--------------------------+-------------------+-------------+
    # | Model                    | CompetitionMath   | GSM8K       |
    # +==========================+===================+=============+
    # | ZhipuAI_chatglm2-6b-base | 25.0 (acc)        | 30.50 (acc) |
    # +--------------------------+-------------------+-------------+
    # | ZhipuAI_chatglm2-6b      | 30.5 (acc)        | 40.50 (acc) |
    # +--------------------------+-------------------+-------------+



# Copyright (c) Alibaba, Inc. and its affiliates.

from typing import List, Union

import pandas as pd
import pyarrow as pa

from llmuses.constants import MetricMembers
from llmuses.utils.arena_utils import compute_elo
from llmuses.utils.logger import get_logger
from llmuses.utils import jsonl_to_list

logger = get_logger()

DEFAULT_COLUMNS_MAPPING = {
    'model_a': 'model_a',
    'model_b': 'model_b',
    'win': 'win',
    'tstamp': 'ts',
    'language': 'lang'
}


class RatingEvaluate(object):

    def __init__(self, metrics: list, baseline_model: str = None, **kwargs):
        self.metrics = metrics
        self.baseline_model = baseline_model
        self.kwargs = kwargs

    def preprocess(self, raw_data_df: pd.DataFrame, **kwargs):

        # Get battles data
        raw_data_df = raw_data_df.sort_values(ascending=True, by=['tstamp'])
        battles = raw_data_df[raw_data_df['anony']].reset_index(drop=True)

        return battles

    def compute_elo_rating(self, raw_data):
        battles = self.preprocess(raw_data_df=raw_data)
        elo_ratings = compute_elo(battles)
        col_model = 'Model'
        col_elo_rating = 'Elo_Rating'
        elo_ratings_res = pd.DataFrame(
            [[n, elo_ratings[n]] for n in elo_ratings.keys()],
            columns=[col_model, col_elo_rating]).sort_values(
                col_elo_rating, ascending=False).reset_index(drop=True)
        elo_ratings_res = elo_ratings_res.round({col_elo_rating: 1})
        return elo_ratings_res

    def get_single_pairwise_rating(self, row: pd.Series):
        tie = False
        if 'win' in row:
            win = row['win']
            if win == 'tie':
                tie = True
            else:
                if win == 'model_a':
                    winner = row['model_a']
                    loser = row['model_b']
                else:
                    winner = row['model_b']
                    loser = row['model_a']
        elif 'win_1' in row:
            win_1 = row['win_1']
            win_2 = row['win_2']
            if win_1 == 'tie' or win_1 != win_2:
                tie = True
            else:
                if win_1 == 'model_a':
                    winner = row['model_a']
                    loser = row['model_b']
                else:
                    winner = row['model_b']
                    loser = row['model_a']
        else:
            raise ValueError('Unsupported data format')

        if tie:
            return [{
                'model': row['model_a'],
                'win': 0,
                'loss': 0,
                'tie': 1
            }, {
                'model': row['model_b'],
                'win': 0,
                'loss': 0,
                'tie': 1
            }]
        else:
            return [{
                'model': winner,
                'win': 1,
                'loss': 0,
                'tie': 0
            }, {
                'model': loser,
                'win': 0,
                'loss': 1,
                'tie': 0
            }]

    def compute_pairwise_rating(self, raw_data):
        df_all = self.preprocess(raw_data_df=raw_data)
        model_list = (
            df_all['model_a'].unique().tolist()
            + df_all['model_b'].unique().tolist())
        model_list = list(set(model_list))

        list_res = []
        # traverse df row by row
        for index, row in df_all.iterrows():
            if self.baseline_model is not None:
                if self.baseline_model not in [row['model_a'], row['model_b']]:
                    logger.warning(
                        f'One of the models in the battle should be the baseline model: {self.baseline_model}'
                    )
                    continue
            rating = self.get_single_pairwise_rating(row)
            list_res = list_res + rating

        df = pd.DataFrame(list_res)
        df = df.groupby(['model']).sum()

        # remove baseline model
        if self.baseline_model is not None:
            df = df[df.index != self.baseline_model]
        # add win rate
        df['win_rate'] = df['win'] / (df['win'] + df['loss'] + df['tie'])
        df['loss_rate'] = df['loss'] / (df['win'] + df['loss'] + df['tie'])
        df['tie_rate'] = df['tie'] / (df['win'] + df['loss'] + df['tie'])
        return df.sort_values(by='win_rate', ascending=False)

    def compute_score_rating(self, raw_data):
        df_all = self.preprocess(raw_data_df=raw_data)
        df = df_all[['model', 'score']]

        df_score = df.groupby(['model']).mean()
        return df_score.sort_values(by='score', ascending=False)

    def eval_samples(self, data_list: list):
        res_all = []

        raw_data: pd.DataFrame = None

        if len(data_list) > 0:
            raw_data = data_list[0]

        for metric in self.metrics:

            if metric == MetricMembers.ELO.value:
                res = self.compute_elo_rating(raw_data)
                res_all.append(res)

            elif metric == MetricMembers.PAIRWISE.value:
                res = self.compute_pairwise_rating(raw_data)
                res_all.append(res)

            elif metric == MetricMembers.SCORE.value:
                res = self.compute_score_rating(raw_data)
                res_all.append(res)

            else:
                raise ValueError(f'Unsupported metric: {metric}')

        return res_all

    def run(self, prompts: Union[str, list], **kwargs) -> List[pd.DataFrame]:
        """
        Load the predicted samples and evaluate them in arena mode.
        """
        # raw_data = pd.read_json(prompts)
        data_list = jsonl_to_list(prompts)
        data_df = pa.Table.from_pylist(data_list).to_pandas()
        res_list = self.eval_samples([data_df])

        return res_list



# Copyright (c) Alibaba, Inc. and its affiliates.
# flake8: noqa

import argparse
import os
from pathlib import Path
import torch
from tqdm import tqdm

from llmuses.constants import EvalConfigKeys
from llmuses.evaluator.rating_eval import RatingEvaluate
from llmuses.models.model_adapter import ChatGenerationModelAdapter
from llmuses.utils import get_obj_from_cfg, yaml_to_dict, jsonl_to_list, dump_jsonl_data
from llmuses.utils.logger import get_logger
from modelscope.utils.hf_util import GenerationConfig

logger = get_logger()

WORK_DIR = Path(__file__).absolute().parent


class ArenaWorkflow:

    def __init__(self, cfg_file: str, **kwargs):

        self.cfg_dict = yaml_to_dict(os.path.join(WORK_DIR, cfg_file))
        logger.info(f'**Arena Config: {self.cfg_dict}')

        self.question_file: str = os.path.join(WORK_DIR, self.cfg_dict.get('question_file'))
        self.answers_gen: dict = self.cfg_dict.get('answers_gen', {})
        self.reviews_gen: dict = self.cfg_dict.get('reviews_gen', {})
        self.reviewer_cfg: dict = ArenaWorkflow._get_obj_from_cfg(self.reviews_gen.get('reviewer', {}))

        self.prompt_file = os.path.join(WORK_DIR, self.reviews_gen.get('prompt_file'))
        self.review_file = os.path.join(WORK_DIR, self.reviews_gen.get('review_file'))

        self.rating_gen: dict = self.cfg_dict.get('rating_gen', {})
        self.report_file: str = os.path.join(WORK_DIR, self.rating_gen.get('report_file'))

    @staticmethod
    def _get_obj_from_cfg(obj_cfg: dict):
        cls_ref = obj_cfg.get(EvalConfigKeys.CLASS_REF, None)
        if not cls_ref:
            logger.warning(
                f'Class reference is not specified in config: {obj_cfg}')
            return obj_cfg

        cls = get_obj_from_cfg(cls_ref)
        obj_cfg[EvalConfigKeys.CLASS_REF] = cls

        return obj_cfg

    def _predict_answers(self,
                         model_id_or_path: str,
                         model_revision: str,
                         precision: torch.dtype,
                         generation_config: GenerationConfig,) -> list:

        # TODO: multi-task to be supported
        model_adapter = ChatGenerationModelAdapter(model_id=model_id_or_path,
                                                   model_revision=model_revision,
                                                   torch_dtype=precision,
                                                   generation_config=generation_config)
        res_list = []
        questions_list = jsonl_to_list(self.question_file)
        for data_d in tqdm(questions_list, total=len(questions_list), desc=f'Predicting(answers):'):
            # {"question_id": 1, "text": "How can I improve my time management skills?", "category": "generic"}
            text = data_d.get('text', None)
            if not text:
                logger.warning(f'Invalid question: {data_d}')
                continue
            prompt = f'Question: {text}\n\nAnswer:'
            inputs = {'data': [prompt]}
            res_d: dict = model_adapter.predict(inputs=inputs)
            ans_text: str = res_d['choices'][0]['message']['content']

            ans = {
                'question_id': data_d['question_id'],
                'text': data_d['text'],
                'category': data_d['category'],
                'model_id': model_id_or_path,
                'metadata': {},
                'answer': ans_text,
            }
            res_list.append(ans)

        return res_list

    def get_answers(self):
        for model_name, cfg_d in self.answers_gen.items():
            enable = cfg_d.get(EvalConfigKeys.ENABLE, True)
            if not enable:
                logger.warning(
                    f'Skip model {model_name} because it is not enabled.')
                continue

            model_id_or_path = cfg_d.get(EvalConfigKeys.MODEL_ID_OR_PATH)
            model_revision = cfg_d.get(EvalConfigKeys.MODEL_REVISION, None)
            precision = cfg_d.get(EvalConfigKeys.PRECISION, torch.float16)
            precision = eval(precision) if isinstance(precision, str) else precision
            generation_config = cfg_d.get(EvalConfigKeys.GENERATION_CONFIG, {})
            generation_config = GenerationConfig(**generation_config)
            ans_output_file = os.path.join(WORK_DIR, cfg_d.get(EvalConfigKeys.OUTPUT_FILE))

            answers_list = self._predict_answers(model_id_or_path=model_id_or_path,
                                                 model_revision=model_revision,
                                                 precision=precision,
                                                 generation_config=generation_config)

            dump_jsonl_data(answers_list, ans_output_file)
            logger.info(f'Answers generated by model {model_name} and saved to {ans_output_file}')

    def get_reviews(self, dry_run: bool = False):
        enable = self.reviews_gen.get(EvalConfigKeys.ENABLE, True)
        if enable:
            reviewer_cls = self.reviewer_cfg.get(EvalConfigKeys.CLASS_REF)
            if not reviewer_cls:
                logger.warning('Skip reviews generation because class reference is not specified.')
                return
            reviewer_args = self.reviewer_cfg.get(EvalConfigKeys.CLASS_ARGS, {})
            target_answers = self.reviews_gen.get('target_answers')
            if target_answers is None:
                # Get all answers from answers_gen config if target_answers is None
                target_answers = [item[EvalConfigKeys.OUTPUT_FILE] for item in self.answers_gen.values()]
            target_answers = [os.path.join(WORK_DIR, item) for item in target_answers]
            target_answers = [file_path for file_path in target_answers if os.path.exists(file_path)]

            baseline_file = self.reviews_gen.get('baseline_file', None)
            if baseline_file:
                baseline_file = os.path.join(WORK_DIR, baseline_file)

            reference_file = self.reviews_gen.get('reference_file', None)
            if reference_file:
                reference_file = os.path.join(WORK_DIR, reference_file)

            cache_file = self.reviews_gen.get('cache_file', None)
            if cache_file:
                cache_file = os.path.join(WORK_DIR, cache_file)

            input_kwargs = dict(
                prompt_file=self.prompt_file,
                answer_file_list=target_answers,
                review_result_file=self.review_file,
                baseline_file=baseline_file,
                reference_file=reference_file,
                reviewer_args=reviewer_args,
                cache_file=cache_file)

            reviewer_obj = reviewer_cls(**input_kwargs)
            reviewer_obj.run(dry_run=dry_run)
            logger.info(f'Reviews with generated by reviewer and saved to {self.review_file}')

        else:
            logger.warning('Skip reviews generation because it is not enabled.')

    def get_rating_results(self):
        enable = self.rating_gen.get(EvalConfigKeys.ENABLE, True)
        if enable:
            report_file = os.path.join(WORK_DIR, self.rating_gen.get('report_file'))
            metrics = self.rating_gen.get('metrics', ['elo'])
            baseline_model = self.rating_gen.get(
                'baseline_model') if metrics[0] == 'pairwise' else None
            ae = RatingEvaluate(metrics=metrics, baseline_model=baseline_model)
            res_list = ae.run(self.review_file)
            rating_df = res_list[0]
            logger.info(f'Rating results:\n{rating_df.to_csv()}')
            rating_df.to_csv(report_file, index=True)
            logger.info(f'Rating results are saved to {report_file}')
        else:
            logger.warning('Skip rating because it is not enabled.')

    def run(self, dry_run: bool = False):

        # Get all answers
        self.get_answers()

        # Get all reviews
        self.get_reviews(dry_run=dry_run)

        # Get rating results
        self.get_rating_results()

        logger.info('*** Arena workflow is finished. ***')


def main():

    # Usage: python llmuses/run_arena.py -c /path/to/xxx_cfg_arena.yaml

    parser = argparse.ArgumentParser(description='LLMs evaluations with arena mode.')
    parser.add_argument('-c', '--cfg-file', required=True)
    parser.add_argument('--dry-run', action='store_true', default=False)
    args = parser.parse_args()

    arena_workflow = ArenaWorkflow(cfg_file=args.cfg_file)
    arena_workflow.run(dry_run=args.dry_run)


if __name__ == '__main__':
    main()



# Copyright (c) Alibaba, Inc. and its affiliates.
# flake8: noqa
import copy
import json
import argparse
from typing import Union, List
import torch        # noqa

from llmuses.config import TaskConfig
from llmuses.constants import DEFAULT_ROOT_CACHE_DIR
from llmuses.evaluator import Evaluator
from llmuses.evaluator.evaluator import HumanevalEvaluator
from llmuses.models.custom import CustomModel
from llmuses.utils import import_module_util, yaml_to_dict
from llmuses.utils.logger import get_logger

logger = get_logger()

"""
Run evaluation for LLMs.
"""

BENCHMARK_PATH_PREFIX = 'llmuses.benchmarks.'
MEMBERS_TO_IMPORT = ['DATASET_ID', 'SUBSET_LIST', 'DataAdapterClass', 'ModelAdapterClass']


def parse_args():
    parser = argparse.ArgumentParser(description='Run evaluation on benchmarks for LLMs.')

    parser.add_argument('--model',
                        help='The model id on modelscope, or local model dir.',
                        type=str,
                        required=True)
    parser.add_argument('--eval-type',
                        type=str,
                        help='The type for evaluating. '
                             'service - for APIs, TO-DO'
                             'checkpoint - for models on ModelScope or local model dir, '
                             'custom - for custom models.'
                             '         Need to set `--model` to llmuses.models.custom.CustomModel format.'
                             'default to `checkpoint`.',
                        required=False,
                        default='checkpoint',
                        )
    parser.add_argument('--model-args',
                        type=str,
                        help='The model args, should be a string.',
                        required=False,
                        default='revision=None,precision=torch.float16,device_map=auto'
                        )
    parser.add_argument('--generation-config',
                        type=str,
                        help='The generation config, should be a string.',
                        required=False,
                        default='do_sample=False,repetition_penalty=1.0,max_new_tokens=512',
                        )
    parser.add_argument('--datasets',
                        help='Dataset id list, align to the module name in llmuses.benchmarks',
                        type=str,
                        nargs='+',
                        required=True)
    parser.add_argument('--dataset-args',
                        type=json.loads,
                        help='The dataset args, should be a json string. The key of dict should be aligned to datasets,'
                             'e.g. {"humaneval": {"local_path": "/to/your/path"}}',
                        required=False,
                        default='{}')
    parser.add_argument('--dataset-dir',
                        help='The datasets dir. Use to specify the local datasets or datasets cache dir.'
                             'See --dataset-hub for more details.',
                        required=False,
                        default=DEFAULT_ROOT_CACHE_DIR)
    parser.add_argument('--dataset-hub',
                        help='The datasets hub, can be `ModelScope` or `HuggingFace` or `Local`. '
                             'Default to `ModelScope`.'
                             'If `Local`, the --dataset-dir should be local input data dir.'
                             'Otherwise, the --dataset-dir should be the cache dir for datasets.',
                        required=False,
                        default='ModelScope')
    parser.add_argument('--outputs',
                        help='Outputs dir. Default to `outputs`, which means dump to current path: ./outputs',
                        required=False,
                        default='outputs')
    parser.add_argument('--work-dir',
                        help='The root cache dir.',
                        required=False,
                        default=DEFAULT_ROOT_CACHE_DIR)
    parser.add_argument('--limit',
                        type=int,
                        help='Max evaluation samples num for each subset. Default to None, which means no limit.',
                        default=None)
    parser.add_argument('--debug',
                        help='Debug mode, will print information for debugging.',
                        action='store_true',
                        default=False)
    parser.add_argument('--dry-run',
                        help='Dry run in single processing mode.',
                        action='store_true',
                        default=False)
    parser.add_argument('--mem-cache',
                        help='To use memory cache or not.',
                        action='store_true',
                        default=False)
    # parser.add_argument('--use-cache',
    #                     help='To reuse the cache or not. Default to True.',
    #                     action='store_true',
    #                     default=True)
    parser.add_argument('--use-cache',
                        help='To reuse the cache or not. Default to `true`.',
                        type=str,
                        default='true')
    parser.add_argument('--stage',
                        help='The stage of evaluation pipeline, '
                             'can be `all`, `infer`, `review`. Default to `all`.',
                        type=str,
                        default='all')

    args = parser.parse_args()

    return args


def parse_str_args(str_args: str) -> dict:
    assert isinstance(str_args, str), 'args should be a string.'
    arg_list: list = str_args.strip().split(',')
    arg_list = [arg.strip() for arg in arg_list]
    arg_dict: dict = dict([arg.split('=') for arg in arg_list])

    final_args = dict()
    for k, v in arg_dict.items():
        try:
            final_args[k] = eval(v)
        except:
            if v.lower() == 'true':
                v = True
            if v.lower() == 'false':
                v = False
            final_args[k] = v

    return final_args


def run_task(task_cfg: Union[str, dict, TaskConfig, List[TaskConfig]]) -> Union[dict, List[dict]]:

    if isinstance(task_cfg, list):
        eval_results = []
        for one_task_cfg in task_cfg:
            eval_results.append(run_task(one_task_cfg))
        return eval_results

    if isinstance(task_cfg, TaskConfig):
        task_cfg = task_cfg.to_dict()
    elif isinstance(task_cfg, str):
        task_cfg = yaml_to_dict(task_cfg)
    elif isinstance(task_cfg, dict):
        logger.info('** Args: Task config is provided with dictionary type. **')
    else:
        raise ValueError('** Args: Please provide a valid task config. **')

    # Get the output task config
    output_task_cfg = copy.copy(task_cfg)
    logger.info(output_task_cfg)

    model_args: dict = task_cfg.get('model_args',
                                    {'revision': 'default', 'precision': torch.float16, 'device_map': 'auto'})
    # Get the GLOBAL default config (infer_cfg) for prediction
    generation_config: dict = task_cfg.get('generation_config',
                                           {'do_sample': False,
                                            'repetition_penalty': 1.0,
                                            'max_length': 2048,
                                            'max_new_tokens': 512,
                                            'temperature': 0.3,
                                            'top_k': 50,
                                            'top_p': 0.8, }
                                           )
    dataset_args: dict = task_cfg.get('dataset_args', {})
    dry_run: bool = task_cfg.get('dry_run', False)
    model: Union[str, CustomModel] = task_cfg.get('model', None)
    eval_type: str = task_cfg.get('eval_type', 'checkpoint')
    datasets: list = task_cfg.get('datasets', None)
    work_dir: str = task_cfg.get('work_dir', DEFAULT_ROOT_CACHE_DIR)
    outputs: str = task_cfg.get('outputs', 'outputs')
    mem_cache: bool = task_cfg.get('mem_cache', False)
    use_cache: bool = task_cfg.get('use_cache', True)
    dataset_hub: str = task_cfg.get('dataset_hub', 'ModelScope')
    dataset_dir: str = task_cfg.get('dataset_dir', DEFAULT_ROOT_CACHE_DIR)
    stage: str = task_cfg.get('stage', 'all')                               # TODO: to be implemented
    limit: int = task_cfg.get('limit', None)
    debug: str = task_cfg.get('debug', False)

    if model is None or datasets is None:
        raise ValueError('** Args: Please provide model and datasets. **')

    model_precision = model_args.get('precision', torch.float16)
    if isinstance(model_precision, str):
        model_precision = eval(model_precision)

    if mem_cache:
        logger.warning('** DeprecatedWarning: `--mem-cache` is deprecated, please use `--use-cache` instead.')

    logger.info(f'** Set use_cache to {use_cache}.')

    # Get model args
    if dry_run:
        from llmuses.models.dummy_chat_model import DummyChatModel
        model_id: str = 'dummy'
        model_revision: str = 'v1.0.0'
    elif eval_type == 'custom':
        model_id: str = None
        model_revision: str = None
    else:
        model_id: str = model
        model_revision: str = model_args.get('revision', None)
        if model_revision == 'None':
            model_revision = eval(model_revision)

    eval_results = dict()
    for dataset_name in datasets:
        # Get imported_modules
        imported_modules = import_module_util(BENCHMARK_PATH_PREFIX, dataset_name, MEMBERS_TO_IMPORT)

        if dataset_name == 'humaneval' and dataset_args.get('humaneval', {}).get('local_path') is None:
            raise ValueError('Please specify the local problem path of humaneval dataset in --dataset-args,'
                             'e.g. {"humaneval": {"local_path": "/to/your/path"}}, '
                             'And refer to https://github.com/openai/human-eval/tree/master#installation to install it,'
                             'Note that you need to enable the execution code in the human_eval/execution.py first.')

        if dry_run:
            from llmuses.models.dummy_chat_model import DummyChatModel
            model_adapter = DummyChatModel(model_cfg=dict())
        elif eval_type == 'custom':
            if not isinstance(model, CustomModel):
                raise ValueError(f'Expected llmuses.models.custom.CustomModel, but got {type(model)}.')
            from llmuses.models.model_adapter import CustomModelAdapter
            model_adapter = CustomModelAdapter(custom_model=model)
        else:
            # Init model adapter
            device_map = model_args.get('device_map', 'auto') if torch.cuda.is_available() else None
            model_adapter = imported_modules['ModelAdapterClass'](model_id=model_id,
                                                                  model_revision=model_revision,
                                                                  device_map=device_map,
                                                                  torch_dtype=model_precision,
                                                                  cache_dir=work_dir)

        if dataset_name == 'humaneval':
            problem_file: str = dataset_args.get('humaneval', {}).get('local_path')

            evaluator = HumanevalEvaluator(problem_file=problem_file,
                                           model_id=model_id,
                                           model_revision=model_revision,
                                           model_adapter=model_adapter,
                                           outputs_dir=outputs,
                                           is_custom_outputs_dir=False, )
        else:
            # TODO: CHECK dataset_args
            dataset_name_or_path: str = dataset_args.get(dataset_name, {}).get('local_path') or imported_modules[
                'DATASET_ID']

            in_prompt_template: str = dataset_args.get(dataset_name, {}).get('prompt_template', '')

            # Init data adapter
            few_shot_num: int = dataset_args.get(dataset_name, {}).get('few_shot_num', None)
            few_shot_random: bool = dataset_args.get(dataset_name, {}).get('few_shot_random', True)
            data_adapter = imported_modules['DataAdapterClass'](few_shot_num=few_shot_num,
                                                                few_shot_random=few_shot_random,
                                                                prompt_template=in_prompt_template,)

            in_subset_list: list = dataset_args.get(dataset_name, {})\
                .get('subset_list', imported_modules['SUBSET_LIST'])
            logger.info(f'\n** Evaluating on subsets for {dataset_name}: {in_subset_list}\n')

            evaluator = Evaluator(
                dataset_name_or_path=dataset_name_or_path,
                subset_list=in_subset_list,
                data_adapter=data_adapter,
                model_adapter=model_adapter,
                use_cache=use_cache,
                root_cache_dir=work_dir,
                outputs_dir=outputs,
                is_custom_outputs_dir=outputs != 'outputs',
                datasets_dir=dataset_dir,
                datasets_hub=dataset_hub,
                stage=stage,
                eval_type=eval_type,
                overall_task_cfg=output_task_cfg,
            )

        infer_cfg = generation_config or {}
        infer_cfg.update(dict(limit=limit))
        res_dict: dict = evaluator.eval(infer_cfg=infer_cfg, debug=debug)

        eval_results[dataset_name] = res_dict

    return eval_results


def main():
    args = parse_args()

    # Get task_cfg
    use_cache: bool = False if args.use_cache.lower() == 'false' else True
    task_cfg = {
        'model_args': parse_str_args(args.model_args),
        'generation_config': parse_str_args(args.generation_config),
        'dataset_args': args.dataset_args,
        'dry_run': args.dry_run,
        'model': args.model,
        'eval_type': args.eval_type,
        'datasets': args.datasets,
        'work_dir': args.work_dir,
        'outputs': args.outputs,
        'mem_cache': args.mem_cache,
        'use_cache': use_cache,
        'dataset_hub': args.dataset_hub,
        'dataset_dir': args.dataset_dir,
        'stage': args.stage,
        'limit': args.limit,
        'debug': args.debug
    }

    run_task(task_cfg)


if __name__ == '__main__':
    # Usage: python3 llmuses/run.py --model ZhipuAI/chatglm2-6b --datasets mmlu hellaswag --limit 10
    # Usage: python3 llmuses/run.py --model qwen/Qwen-1_8B --generation-config do_sample=false,temperature=0.0 --datasets ceval --dataset-args '{"ceval": {"few_shot_num": 0}}' --limit 10
    main()



# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os

import datasets
import pandas as pd


_CITATION = """\
@inproceedings{lai-etal-2017-race,
    title = "{RACE}: Large-scale {R}e{A}ding Comprehension Dataset From Examinations",
    author = "Lai, Guokun  and
      Xie, Qizhe  and
      Liu, Hanxiao  and
      Yang, Yiming  and
      Hovy, Eduard",
    booktitle = "Proceedings of the 2017 Conference on Empirical Methods in Natural Language Processing",
    month = sep,
    year = "2017",
    address = "Copenhagen, Denmark",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/D17-1082",
    doi = "10.18653/v1/D17-1082",
    pages = "785--794",
}
"""

_DESCRIPTION = """\
RACE is a large-scale reading comprehension dataset with more than 28,000 passages and nearly 100,000 questions.
"""

_HOMEPAGE = "https://modelscope.cn/datasets/modelscope/race/summary"

_URL = "https://modelscope.cn/api/v1/datasets/modelscope/race/repo?Revision=master&FilePath=race.zip"

task_list = [
    "high",
    "middle",
]


class RACEConfig(datasets.BuilderConfig):
    def __init__(self, **kwargs):
        super().__init__(version=datasets.Version("1.0.0"), **kwargs)


class RACE(datasets.GeneratorBasedBuilder):
    BUILDER_CONFIGS = [
        RACEConfig(
            name=task_name,
        )
        for task_name in task_list
    ]

    def _info(self):
        features = datasets.Features(
            {
                "example_id": datasets.Value("string"),
                "article": datasets.Value("string"),
                "answer": datasets.Value("string"),
                "question": datasets.Value("string"),
                "options": [datasets.Value("string")],
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        data_dir = dl_manager.download_and_extract(_URL)
        task_name = self.config.name
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": os.path.join(
                        data_dir, f"race/test/{task_name}-00000-of-00001.parquet"
                    ),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": os.path.join(
                        data_dir, f"race/val/{task_name}-00000-of-00001.parquet"
                    ),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(
                        data_dir, f"race/train/{task_name}-00000-of-00001.parquet"
                    ),
                },
            ),
        ]

    def _generate_examples(self, filepath):
        df = pd.read_parquet(filepath)
        df.columns = ["example_id", "article", "answer", "question", "options"]

        for i, instance in enumerate(df.to_dict(orient="records")):
            yield i, instance


# Copyright (c) Alibaba, Inc. and its affiliates.

import logging
from collections import defaultdict
from pathlib import Path
from statistics import mean

from tqdm import tqdm

from llmuses.constants import MetricsConstant
from llmuses.metrics.bundled_rouge_score import rouge_scorer
from llmuses.preprocess.tokenizers.gpt2_tokenizer import DummyTokenizer
from rouge_chinese import Rouge
import jieba

HERE = Path(__file__).absolute().parent

logger = logging.getLogger(__name__)

scorer = rouge_scorer.RougeScorer(['rouge1', 'rouge2', 'rougeL'],
                                  tokenizer=DummyTokenizer())
zh_scorer = Rouge()


def is_contains_chinese(strs):
    for _char in strs:
        if '\u4e00' <= _char <= '\u9fa5':
            return True
    return False

def compute_rouge_score(predict_l, reference_l):
    assert len(predict_l) == len(reference_l)
    if len(predict_l) == 0:
        tmp_d = dict()
        for key in MetricsConstant.ROUGE_KEYS:
            tmp_d[key] = 0
        return tmp_d

    result = defaultdict(list)
    for p, r in tqdm(zip(predict_l, reference_l)):
        one_sample = compute_rouge_score_one_sample(p, r)
        for rouge_key in MetricsConstant.ROUGE_KEYS:
            result[rouge_key].append(one_sample[rouge_key])
    rlt = {}
    for rouge_key in MetricsConstant.ROUGE_KEYS:
        rlt[rouge_key] = mean(result[rouge_key]) * 100 if rouge_key in result \
            else MetricsConstant.INVALID_VALUE
    return rlt

def compute_rouge_score_one_sample_zh(predict, reference):
    result = dict()
    for p, r in zip(predict, reference):
        p = ' '.join(jieba.cut(p)) if is_contains_chinese(p) else p
        r = ' '.join(jieba.cut(r)) if is_contains_chinese(r) else r

        score = zh_scorer.get_scores(p, r)[0]
        result['rouge-1-r'] = score['rouge-1']['r']
        result['rouge-1-p'] = score['rouge-1']['p']
        result['rouge-1-f'] = score['rouge-1']['f']
        result['rouge-2-r'] = score['rouge-2']['r']
        result['rouge-2-p'] = score['rouge-2']['p']
        result['rouge-2-f'] = score['rouge-2']['f']
        result['rouge-l-r'] = score['rouge-l']['r']
        result['rouge-l-p'] = score['rouge-l']['p']
        result['rouge-l-f'] = score['rouge-l']['f']
    
    return result

def compute_rouge_score_one_sample(predict, reference):
    result = dict()
    for p, r in zip(predict, reference):
        score = scorer.score(p, r)
        result['rouge-1-r'] = score['rouge1'].recall
        result['rouge-1-p'] = score['rouge1'].precision
        result['rouge-1-f'] = score['rouge1'].fmeasure
        result['rouge-2-r'] = score['rouge2'].recall
        result['rouge-2-p'] = score['rouge2'].precision
        result['rouge-2-f'] = score['rouge2'].fmeasure
        result['rouge-l-r'] = score['rougeL'].recall
        result['rouge-l-p'] = score['rougeL'].precision
        result['rouge-l-f'] = score['rougeL'].fmeasure

    return result


def _to_table(final_result) -> str:
    table = []
    # step 1. table header
    all_tasks = ['', 'total']
    all_tasks.extend(final_result['all_tasks'].split(','))
    table.append('\t'.join(all_tasks))

    # step 2. table row
    for rouge_key in MetricsConstant.ROUGE_KEYS:
        row = [rouge_key]
        for task in all_tasks:
            if not task:
                continue
            elif task == 'total':
                row.append(
                    f'{final_result["total"]["rouge"][rouge_key] :0.2f}')
            else:
                row.append(
                    f'{final_result["tasks"][task]["rouge"][rouge_key] :0.2f}')
        table.append('\t'.join(row))

    return '\n'.join(table)


def run_rouge_eval(data_l, md_level=2, report_metric_key='rouge-l-f'):
    print(f"{'#' * md_level} Rouge Eval")
    for data in tqdm(data_l):
        data['rouge'] = compute_rouge_score_one_sample(
            data['gen_tok_str'], data['reference_tok_str'])
    task_data_d = defaultdict(list)
    for data in data_l:
        for task in data['task_tags']:
            task_data_d[task].append(data)

    total_rouge = mean([data['rouge'][report_metric_key] for data in data_l])
    print(f'[total], count: {len(data_l)}, {report_metric_key}: '
          f'{total_rouge * 100:0.2f}%')

    for task, task_data in task_data_d.items():
        task_rouge = mean(
            [data['rouge'][report_metric_key] for data in task_data])
        print(
            f'[{task}], count: {len(task_data_d[task])}, {report_metric_key}: '
            f'{task_rouge * 100:0.2f}%')
